kPhaseEmPkgName = "PHASEEM"

kBicScore = as.integer(1)
kAicScore = as.integer(2)
kLlkScore = as.integer(3)

# TODO : determine what 'maxDrop' is supposed to be at each instance and write
# a function that retrieves it

BootstrapAncDbs = function(x) {
  for (k in 1:length(x)) {
    nr = nrow(x[[k]])
    i = sample(1:nr, nr, replace = TRUE)
    x[[k]] = x[[k]][i,]
  }
  return(x)
}

ReverseSnpOrd = function(x) {
  if (is.list(x)) {
    for (k in 1:length(x)) {
      x[[k]] = ReverseSnpOrd(x[[k]])
    }
  } else if (length(dim(x)) == 2) {
    i = rev(1:ncol(x))
    x = x[,i]
  } else {
    x = rev(x)
  }
  return(x)
}

ExpNumPop1 = function(ancPrbs) {
  print("ExpNumPop1")
  nGroups = dim(ancPrbs)[1]

  if (length(dim(ancPrbs)) == 4) {
    z = 2 * ancPrbs[1,1,,]
    for (k in 2:nGroups) {
      z = z + ancPrbs[1,k,,] + ancPrbs[k,1,,]
    }
  } else {
    z = 2 * ancPrbs[1,1,]
    for (k in 2:nGroups) {
      z = z + ancPrbs[1,k,] + ancPrbs[k,1,]
    }
  }
  return(z)
}

AllNegativeCoding = function(x) {
  print("AllNegativeCoding")
  if (is.list(x)) {
    y = vector(mode = "list")
    for (k in 1:length(x)) {
      y[[k]] = AllNegativeCoding(x[[k]])
    }
    return(y)
  }

  i = (x >= 0)
  if (sum(i) > 0) {
    x[i] = -x[i] - as.integer(1)
  }
  return(x)
}

AllPositiveCoding = function(x) {
  if (is.list(x)) {
    y = vector(mode = "list")
    for (k in 1:length(x)) {
      y[[k]] = AllPositiveCoding(x[[k]])
    }
    return(y)
  }

  i = (x < 0)
  if (sum(i) > 0) {
    x[i] = -x[i] - as.integer(1)
  }
  return(x)
}

AradmxControl = function(modelOrd = NULL, tiedStruct = NULL,
                         jeffreyPrior = TRUE) {
  r = list(modelOrd = 15,
           revSnpOrd = FALSE,
           randParents = FALSE,
           pCount = 1,
           scoreType = kBicScore,
           tiedStruct = TRUE,
           maxNumSplits = 5, 
           minNodeSize = 5,
           splitCritPcount = 1,
           splitOnPops = FALSE,
           popWtdScore = FALSE,
           newVersion = TRUE,
           sampleSplits = FALSE,
           addCanaries = FALSE,
           bootStrap = FALSE,
           bootStrapCounts = TRUE,
           crossValidate = FALSE,
           keepForDebugging = FALSE)
  r$perSnpMdlOrds = NULL
  
  if (jeffreyPrior) {
    r$pCount = 0.5
    r$splitCritPcount = 0.5
  }
  
  if (!is.null(modelOrd)) {
    r$modelOrd = modelOrd
  }
  if (!is.null(tiedStruct)) {
    r$tiedStruct = tiedStruct
  }
  return(r)
}

AradmxPhasedStateMat = function(nGroups, winSz) {
  nr = nGroups * (winSz + 1)
  stateMat = matrix(nrow = nr, ncol = 2, data = as.integer(0))
  ri = 1
  for (g in 0:(nGroups-1)) {
  for (j in 0:winSz) {
    stateMat[ri,] = c(j, g)
    ri = ri + 1
  }}
  
  EltsCompat = function(l, r) {
    if (l[1] == 0 || l[1] == winSz) {
      if (r[1] != 0 && r[1] != 1) return(FALSE) 
      if (r[1] == 0 && r[2] != l[2]) return(FALSE)
    } else {
      if (r[1] != l[1] + 1) return(FALSE)
      if (r[2] != l[2]) return(FALSE)
    }
    return(TRUE)
  }
  nCompat = 0
  for (j in 1:nr) {
  for (k in 1:nr) {
    if (EltsCompat(stateMat[j,], stateMat[k,])) nCompat = nCompat + 1
  }}
  
  i1 = as.integer(1)
  
  L2RMap = rep(as.integer(0), nCompat + nr)
  idx = 1
  for (j in 1:nr) {
    nc = 0
    idx2 = idx + 1
    for (k in 1:nr) {
      if (EltsCompat(stateMat[k,], stateMat[j,])) {
        nc = nc + i1
        L2RMap[idx2] = k - i1
        idx2 = idx2 + 1
      }
    }
    L2RMap[idx] = nc
    idx = idx + 1 + nc
  }
  
  R2LMap = rep(as.integer(0), nCompat + nr)
  idx = 1
  for (j in 1:nr) {
    nc = 0
    idx2 = idx + 1
    for (k in 1:nr) {
      if (EltsCompat(stateMat[j,], stateMat[k,])) {
        nc = nc + i1
        R2LMap[idx2] = k - i1
        idx2 = idx2 + 1
      }
    }
    R2LMap[idx] = nc
    idx = idx + 1 + nc
  }

  stateMat = t(stateMat)
  stateMat = matrix(nrow = nrow(stateMat), data = as.integer(stateMat))
  L2RMap = as.integer(L2RMap)                            
  R2LMap = as.integer(R2LMap)
  r2 = list(stateMat = stateMat, L2RMap = L2RMap, R2LMap = R2LMap)
  return(r2)
}

AradmxBnUnphStateMat = function(nGroups, winSz, reduced = FALSE) {
  r = vector(mode = "list", length = 3)
  .Call("UnphApproxStatesSxp_RI", nGroups, winSz, FALSE, r,
        PACKAGE = kPhaseEmPkgName)
  r2 = list(stateMat = r[[1]],
            L2RMap = r[[2]],
            R2LMap = r[[3]])
  if (reduced) {
    v = vector(mode = "list", length = 3)
    .Call("ReduceUnphApproxStatesSxp_RI", r2$stateMat,
          r2$L2RMap, r2$R2LMap, v,
          PACKAGE = kPhaseEmPkgName)
    r2 = list(stateMat = v[[1]],
              L2RMap = v[[2]],
              R2LMap = v[[3]])
  }

  return(r2)
}

