FlipAlleles = function(xA0, xA1, yA0, yA1) {
  f = function(x) {
    y = x;
    y[x == "A"] = "T"
    y[x == "T"] = "A"
    y[x == "G"] = "C"
    y[x == "C"] = "G"
    return(y)
  }
  
  revI1 = ((xA0 == yA1) & (xA1 == yA0))
  revI2 = ((xA0 == f(yA1)) & (xA1 == f(yA0)))
  
  matchI1 = ((xA0 == yA0) & (xA1 == yA1))
  matchI2 = ((xA0 == f(yA0)) & (xA1 == f(yA1)))
  
  okI = (matchI1 | matchI2)
  revI = (revI1 | revI2)
  
  ambigI = (okI & revI) 
  okI = okI & !ambigI
  revI = revI & !ambigI
  misMatchI = !(okI | revI | ambigI)
  
  return(list(okI = okI, revI = revI, ambigI = ambigI,
              misMatchI = misMatchI))
}

ReadPedFile = function(pedPath, bimPath = NULL, bimFile = NULL) {
  if (!is.null(bimPath)) {
    bimFile = read.table(file = bimPath, as.is = TRUE, header = FALSE)
  }

  nSnp = nrow(bimFile)
  sampIds = c()

  con = file(pedPath, "r")
  while (isOpen(con)) {
    x = scan(con, what=character(), nlines = 1, sep = " ", quiet = TRUE)
    if (length(x) == 0) {
      break
    }

    sampIds = c(sampIds, x[2])
    
    if (nSnp * 2 != length(x) - 6) {
      stop(paste("inconsistent number of columns in row", length(sampIds)))
    }
    print(length(sampIds))
  }
  close(con)

  nSamp = length(sampIds)
  aMiss = BinaryMat(nSnp, nSamp * 2)
  aMat = BinaryMat(nSnp, nSamp * 2)

  a0 = rep(bimFile[, 5], each = 2)
  a1 = rep(bimFile[, 6], each = 2)

  con = file(pedPath, "r")

  i2 = seq(from = 2, to = (2 * nSnp), by = 2)
  i1 = i2 - 1

  for (k in 1:nSamp) {
    print(k)
    x = scan(con, what=character(), nlines = 1, sep = " ", quiet = TRUE)
    x = x[-(1:6)]

    y0 = (x == a0)
    y1 = (x == a1)

    aMiss[, 2 * k] = (!(y0 | y1))[i1]
    aMiss[, 2 * k + 1] = (!(y0 | y1))[i2]
    aMat[, 2 * k] = y1[i1]
    aMat[, 2 * k + 1] = y1[i2]
  }
  close(con)

  return(list(sampIds = sampIds, bimFile = bimFile,
              aMat = aMat, aMiss = aMiss))
}


HaploRec.output <- function(db, file.path, scramble=T, verbose = F)
{        
  if(scramble){
    db <- RandPhasing(db)
  }
  
  nSnp = ncol(db)
  
  header.str = rep("", nSnp+1)
  header.str[1] = "Id"
  for(j in 2:length(header.str)){
    header.str[j] = paste("M",j, sep="")
  }
  
	write(header.str, sep=" ", ncolumns = (nSnp+1), file=file.path)
	i1 <- seq(from=2, to=nrow(db), by=2) - 1

  t1 = proc.time()[[3]]
  
	for(k in i1){
		v1 <- db[k,]+as.integer(1)
		v2 <- db[k+1,]+as.integer(1)

		if(sum(v1 > 1) > 1){
			v1[v1 > 1] <- as.integer(0)   # 0 means missing
		}
		if(sum(v2 > 1) > 1){
			v2[v2 > 1] <- as.integer(0)
		}
		v1 = c(k, v1)  # we have to append the sample ID to the first column
		v2 = c(k, v1)
		write(v1, sep=" ", append=T, file=file.path, ncolumns = (nSnp+1))
		write(v2, sep=" ", append=T, file=file.path, ncolumns = (nSnp+1))
		
		if(proc.time()[[3]] - t1 > 5 && verbose ){
		  print(k)
		  t1 = proc.time()[[3]]
		}
	}
}

BEAGLE.input = function(filePath) {
  x = read.table(file=filePath, header=FALSE, sep=" ")
  x = as.matrix(x[-1, -(1:2)])
  x = matrix(nrow=nrow(x), data=as.integer(x))
  return(t(x))
}

BEAGLE.DummySnpIds = function(n) {
  snpStr = rep("", n)
  for (k in 1:n) {
    snpStr[k] = paste("SNP",k, sep="")
  }
  return(snpStr)
}

BEAGLE.FormatMatrix = function(db, affectStat = NULL, rs = NULL) {
  z = matrix(nrow=nrow(db), data=as.character(db))
  z[db == 2] = "?"
  z = t(z)

  if (is.null(rs)) {
    snpStr = BEAGLE.DummySnpIds(nrow(z))
  } else {
    snpStr = rs
  }

  z = cbind(rep("M", nrow(z)), snpStr, z)
  
	header.str = c("I", "id")
  v1 = rep(1:(nrow(db)/2), each=2)
  for (j in v1) header.str = c(header.str, toString(j))
  
  if (!is.null(affectStat)) {
    if (!is.list(affectStat)) {
      affectStat = list(trait = affectStat)
    }
    
    for (k in 1:length(affectStat)) {
      aff.str = c("A", names(affectStat)[k])
      v = affectStat[[k]]
      for (j in 1:length(v)) {
        aff.str = c(aff.str, toString(v[j]))
      }
      z = rbind(aff.str, z)
    }
  }    
  z = rbind(header.str, z)
  return(z)
}

BEAGLE.output = function(db, file.path, scramble=TRUE, verbose = FALSE,
                         affectStat = NULL, rs = NULL) {
  if (scramble) {
    db = RandPhasing(db)
  }
  z = BEAGLE.FormatMatrix(db, affectStat, rs)
  write.table(z, file=file.path, quote = FALSE, sep = " ",
              row.names = FALSE, col.names = FALSE)
  return(z)
}

BEAGLE.wrapper = function(db, pathBase, phasedRefPanel = NULL, rs = NULL,
                          removeIntermediate = TRUE, lowmem = "false",
                          gprobs = "false", tmpDir = NULL,
                          callPrefix = "", dontRun = FALSE,
                          readOnly = FALSE) {
  nHap = nrow(db)
  
  inpFname = paste(pathBase, ".unphInp", sep = "")
  inpMat = BEAGLE.output(db, inpFname, rs = rs)

  javaArgs = "-Xmx1000m -jar beagle.jar"
  bglArgs = sprintf("unphased=%s missing=? log=%s.log lowmem=%s gprobs=%s",
                    inpFname, pathBase, lowmem, gprobs)
                    
  phasedRefPanelFnm = NULL
  markersFnm = NULL
  
  if (!is.null(phasedRefPanel)) {
    phasedRefPanelFnm = paste(pathBase, ".refPanel", sep = "")
    
    if (!readOnly) {
      BEAGLE.output(phasedRefPanel, phasedRefPanelFnm, scramble = FALSE, rs = rs)
    }
    bglArgs = paste(bglArgs, " phased=", phasedRefPanelFnm, sep = "")
    
    if (is.null(rs)) {
      snpStr = BEAGLE.DummySnpIds(nrow(inpMat))
    } else {
      snpStr = rs
    }    
    markersFnm = paste(pathBase, ".markers", sep = "")
    markersMat = cbind(snpStr, 1:length(snpStr),
                       rep(0, length(snpStr)),
                       rep(1, length(snpStr)))
                       
    if (!readOnly) {
      write.table(markersMat, file=markersFnm, quote = FALSE, sep = " ",
                  row.names = FALSE, col.names = FALSE)
    }
    bglArgs = paste(bglArgs, " markers=", markersFnm, sep = "")
    
  }
  
  if (!is.null(tmpDir)) {
    javaArgs = paste("-Djava.io.tmpdir=", tmpDir, " ", javaArgs, sep = "")
  }

  bglCmd = paste(callPrefix, "java ", javaArgs, " ", bglArgs, sep="")
  if (dontRun) {
    return(bglCmd)
  } else if (!readOnly) {
    system(bglCmd)
  }

  bglOut = paste(inpFname, ".phased", sep="")
  haps = BEAGLE.input(bglOut)
  if (removeIntermediate) {
    system(paste("rm", bglOut))
    system(paste("rm", inpFname))
    if (!is.null(phasedRefPanelFnm)) system(paste("rm", phasedRefPanelFnm))
    if (!is.null(markersFnm)) system(paste("rm", markersFnm))
  }
  return(haps)
}

BEAGLE.output.old = function(db, file.path, scramble=T, verbose = F) {

  if (scramble) {
    db = RandPhasing(db)
  }
  
  nCase = nrow(db)
  nSamp = nCase/2
  
  outNCol = nCase + 2
  
	header.str = c("I", "id")
  v1 = rep(1:nSamp, each=2)
  for (j in v1) header.str = c(header.str, toString(j))
  
  write(paste(header.str, sep="\t"), file=file.path, ncolumns=outNCol)
  
  #affect.stat = c("A", "NA", rep("0", nSamp) )   
  #write(paste(affect.stat, sep="\t"), append=T, file=file.path, ncolumns=outNCol)

  t1 = proc.time()[[3]]
  
  ncDb = ncol(db)
  
  curStr = c()
  curStrNchar = 0
	for (k in 1:ncDb) {
	  if (verbose && proc.time()[[3]] > t1 + 5) {
	    t1 = proc.time()[[3]]
	    print(k)
    }
	  mark.str = c("M", paste("SNP",k, sep=""), db[,k])
	  
	  i1 = which(db[,k] > 1)
	  if (length(i1) > 0) {
	    mark.str[i1+2] = "?"
    }
    
    mark.str = paste(mark.str, sep="\t")
#    curStrNchar = curStrNchar + nchar(mark.str)
#    curStr = paste( curStr, mark.str, sep="\n")
#    
#    if (curStrNchar > batchSz || k == ncDb) {
#      write(curStr, append=T, file=file.path, ncolumns=outNCol)
#      curStrNchar = 0
#      curStr = c()
#    }
     
		write(mark.str, append=T, file=file.path, ncolumns=outNCol)
	}
}

fastPhase.output = function(x, file.path, scramble=TRUE) {
  # This function writes x to file in a format that fastPHASE
  # can read.  If 'scramble' is TRUE, we randomize the phasing
  # before writing.  The file format uses question marks (?)
  # for missing alleles (x[i,j] == 2)
  if (scramble) {
    x = RandPhasing(x)
  }
  if ((nrow(x) %% 2) != 0) {
    stop("x must have even number of rows")
  }
  
  M = nrow(x) / 2
  N = ncol(x)
	header.str = paste(nrow(x)/2, N, sep="\n")
	write(header.str, file=file.path)
	i1 = seq(from=2, to=(2*M), by=2) - 1

	for (k in i1) {
		v1 = x[k,]
		v2 = x[k+1,]
		write(paste("# id", (k+1)/2), append=TRUE, file=file.path)

		if (sum(v1 > 1) > 0) {
			v1[v1 > 1] = "?"
		}
		if (sum(v2 > 1) > 0) {
			v2[v2 > 1] = "?"
		}
		write(v1, sep="", append=TRUE, file=file.path, ncolumns=N)
		write(v2, sep="", append=TRUE, file=file.path, ncolumns=N)
	}
}

fastPhase.input = function(file.path, n.samp) {
	ffLines = read.table(file=file.path, header=FALSE, as.is = T, sep = "$",
                        comment.char="")
	
	retMat = NULL
	j = 1
	while(j <= nrow(ffLines)) {
    firstChar = substr(ffLines[j,1], 1, 1)
    if (firstChar == "#") {
      j = j + 1
      h1 = as.integer( strsplit(ffLines[j,1], " ")[[1]])
      j = j + 1
      h2 = as.integer( strsplit(ffLines[j,1], " ")[[1]])
      
      if (is.null(retMat)) {
        retMat = rbind(h1, h2)
      } else {
        retMat = rbind(retMat, h1, h2)
      }
    } else {
      j = j + 1
    }
  }
	return(retMat)
}

fastPhase.input.aschar = function(file.path, n.samp) {
	ffLines = read.table(file=file.path, header=F, as.is = T, sep = "$",
                        comment.char="")
	
	retMat = NULL
	j = 1
	while(j <= nrow(ffLines)) {
    firstChar = substr(ffLines[j,1], 1, 1)
    if (firstChar == "#") {
      j = j + 1
      h1 = strsplit(ffLines[j,1], " ")[[1]]
      j = j + 1
      h2 = strsplit(ffLines[j,1], " ")[[1]]
      
      if (is.null(retMat)) {
        retMat = rbind(h1, h2)
      } else {
        retMat = rbind(retMat, h1, h2)
      }
    } else {
      j = j + 1
    }
  }
	return(retMat)
}