AradmxBuildTrees = function(x, popIdx = 1, controlObj = NULL, popLabels = NULL) {
  if (is.null(controlObj)) {
    controlObj = AradmxControl()
  }
  p = controlObj
  
  # returns vectors of parent sets of decreasing model order.  If 'r' is the returned
  # vector, then r[[k+1]] contains a model of order half that of r[[k]]
  # The model order is halved until it reaches zero.
  #
  # r[[ modelOrder ]][[ 1 ]] contains parent sets in packed format
  #     (see BnPhPackPsetsBinTree in K2BinTree.c for documentation)
  # r[[ modelOrder ]][[ 2 ]] contains the tree structure itself
  # r[[ modelOrder ]][[ 3 ]] contains the counts at the leaf nodes
  # r[[ modelOrder ]][[ 4 ]] is the model order
  #
  #
  
  if (is.null(popLabels)) {
    popLabels = rep(as.integer(1), nrow(x))
  }
  
  splitPenalty = 0
  if (p$scoreType == kBicScore) {
    splitPenalty = -0.5 * log(nrow(x))
    if (p$popWtdScore) splitPenalty = splitPenalty * (1 + max(popLabels))
  } else if (p$scoreType == kAicScore) {
    splitPenalty = -2
  }
  allowedParents = NULL

  nSnp = ncol(x)

  edgeWidths = c()
  r = vector(mode = "list")

  maxEdgeWidth = p$modelOrd
  
  while (maxEdgeWidth >= 0) {
    splitLists = vector(mode = "list", length = nSnp)
    leafCounts = vector(mode = "list", length = nSnp)
    
    verbose = TRUE

    if (is.null(popLabels)) {
      stop("popLabels is NULL.  new version requires it.")
    }
    popLabels0 = as.integer(popLabels-1)
    
    if (min(popLabels0) < 0) {
      stop("input popLabels must be 1, 2, .... The are converted to 0,1, ... for the C code")
    }
    if (is.null(p$perSnpMdlOrds)) {
      maxEdgeWidths = rep(maxEdgeWidth, ncol(x))
    } else {
      maxEdgeWidths = pmin(p$perSnpMdlOrds, maxEdgeWidth)
    }
    maxEdgeWidths = as.integer(maxEdgeWidths)
    
    .Call("GrowBinTrees_RI", as.BinaryMat(t(x)), popLabels0,
					maxEdgeWidths, p, splitPenalty, as.integer(popIdx-1),
          verbose, splitLists, leafCounts,
          PACKAGE = kPhaseEmPkgName)
    r[[length(r)+1]] = list(splitLists = splitLists, leafCounts = leafCounts,
                            maxEdgeWidth = maxEdgeWidth)

    if (maxEdgeWidth == 0) {
      break
    } else if (maxEdgeWidth == 3) {
      maxEdgeWidth = 2
    } else {
      maxEdgeWidth = max(as.integer(maxEdgeWidth/2), maxEdgeWidth - 3)
    }
  }
  return(r)
}