# This is our way of dealing with data that is too big to fit into memory
# 

kPhaseEmPkgName = "PHASEEM"

DeepCopyDbl = function(x) {
  return(x + 0.0)
}

FactoredMatrix = function(nr, nc, nfact) {
  u = matrix(nrow = nr, ncol = nfact, data = as.double(0)) 
  v = matrix(nrow = nfact, ncol = nc, data = as.double(0))
  r = list(u = u, v = v, nr = as.integer(nr), nc = as.integer(nc),
           nfact = as.integer(nfact),
           type = "FactoredMatrix")
  class(r) = "FactoredMatrix"
  return(r)
}

as.double.FactoredMatrix = function(x) {
  return(x$u %*% x$v)
}

t.FactoredMatrix = function(x) {
  x2 = list(u = x$v, v = x$u, nr = x$nc, nc = x$nr, nfact = x$nfact,
            type = "FactoredMatrix")
  class(x2) = "FactoredMatrix"
}

"[.FactoredMatrix" = function(x, subset = NULL, select = NULL) {
  if (is.null(subset)) subset = 1:nrow(x)      
  if (is.null(select)) select = 1:ncol(x)
  u = x$u[subset, ]
  v = x$v[, select]
  x2 = list(u = u, v = v, nr = nrow(u), nv = ncol(v), nfact = x$nfact,
            type = "FactoredMatrix")
  class(x2) = "FactoredMatrix"
}

OnesMat = function(nr, nc) {
  r = list(nr = as.integer(nr), nc = as.integer(nc), type = "OnesMat")
  class(r) = "OnesMat"
  return(r)
}

t.OnesMat = function(x) {
  nc = x$nc
  x$nc = x$nr
  x$nr = nc
  return(x)
}

as.integer.OnesMat = function(x) {
  return(matrix(nrow = x$nr, ncol = x$nc, data = as.integer(1)))
}

BinaryMat = function(nr, nc, nri = NULL) {
  if (is.null(nri)) nri = as.integer(ceiling(nr / 32))
  m = matrix(nrow = nri, ncol = nc, data = as.integer(0))
  r = list(dat = m, nr = as.integer(nr), nc = as.integer(nc),
           nri = nri, type = "BinaryMat",
           is_binary_mat = TRUE)
  class(r) = "BinaryMat"
  return(r)
}

BinaryMatScaled = function(nr, nc, nri = NULL) {
  if (is.null(nri)) nri = as.integer(ceiling(nr / 32))
  m = matrix(nrow = nri, ncol = nc, data = as.integer(0))
  r = list(dat = m, nr = as.integer(nr), nc = as.integer(nc),
           rmlt = rep(1.0, nr),                           
           cmlt = rep(1.0, nc),
           rshift = rep(0.0, nr),                           
           cshift = rep(0.0, nc),
           nri = nri, type = "BinaryMatScaled",
           is_binary_mat = TRUE)
  class(r) = "BinaryMatScaled"
  return(r)
}

is.BinaryMatScaled = function(x) {
  return(class(x) == "BinaryMatScaled")
}

as.BinaryMatScaled = function(x) {
  r = as.BinaryMat(x)
  nr = nrow(r)
  nc = ncol(r)
  if (is.BinaryMatScaled(x)) {
    r$rmlt = DeepCopyDbl(x$rmlt)                          
    r$cmlt = DeepCopyDbl(x$cmlt) 
    r$rshift = DeepCopyDbl(x$rshift)                            
    r$cshift = DeepCopyDbl(x$cshift) 
  } else {
    r$rmlt = rep(1.0, nr)                           
    r$cmlt = rep(1.0, nc)
    r$rshift = rep(0.0, nr)                           
    r$cshift = rep(0.0, nc)
  }
  r$type = "BinaryMatScaled"
  class(r) = "BinaryMatScaled"
  return(r)
}

ByteMat = function(nr, nc, nri = NULL) {
  if (is.null(nri)) nri = as.integer(ceiling(nr / 4))
  m = matrix(nrow = nri, ncol = nc, data = as.integer(0))
  r = list(dat = m, nr = as.integer(nr), nc = as.integer(nc),
           nri = nri, type = "ByteMat",
           is_byte_mat = TRUE)
  class(r) = "BinaryMat"
  return(r)
}

MatrixHash = function(x) {
  return(.Call("MatrixHash", x, PACKAGE = kPhaseEmPkgName))
}

MatrixNrow = function(x) {
  if (class(x) == "BinaryMat") {
    return(x$nr)
  } else {
    return(nrow(x))
  }
} 

MatrixNcol = function(x) {
  if (class(x) == "BinaryMat") {
    return(x$nc)
  } else {
    return(ncol(x))
  }
}

MatrixDim = function(x) {
  return(c(MatrixNrow(x), MatrixNcol(x)))
}

MatrixType = function(x) {
  if (is.integer(x)) {
    return("IntMat")
  } else if (is.double(x)) {
    return("DblMat")
  } else if (is.ByteMat(x)) {
    return("ByteMat")
  } else if (is.BinaryMat(x)) {
    return("BinaryMat")
  } else {
    stop("unknown type")
  }
  return("?")
}

MatrixFill = function(x, val) {
  # modifies 'x' in place
  return(.Call("FillMatrix", val, x, PACKAGE = kPhaseEmPkgName))
}

MatrixRowSums = function(x) {
  return(.Call("MarginSums", x, 1, NULL, PACKAGE = kPhaseEmPkgName))
}

MatrixColSums = function(x) {
  return(.Call("MarginSums", x, 2, NULL, PACKAGE = kPhaseEmPkgName))
}
   
MatrixCompMult = function(x, y, matType = NULL) {
  # component-wise multiplication
  if (is.null(matType)) {
    matType = class(x)
  }
  z = AllocMatrix(nrow(x), ncol(x), matType)
  return(.Call("MatrixCompMult", x, y, z, PACKAGE = kPhaseEmPkgName))
}

MatrixNegate = function(x, matType = NULL) { 
  if (is.null(matType)) {
    matType = class(x)
  }
  y = AllocMatrix(nrow(x), ncol(x), matType)
  return(.Call("MatrixNegate", x, y, PACKAGE = kPhaseEmPkgName))
}

AllocMatrix = function(nr, nc, matType = "IntMat") {  
  if (matType == "BinaryMat") {
    z = BinaryMat(nr, nc)
  } else if (matType == "BinaryMatScaled") {
    z = BinaryMatScaled(nr, nc)
  } else if (matType == "ByteMat") {
    z = ByteMat(nr, nc)
  } else if (matType == "IntMat") {
    z = matrix(nrow = nr, ncol = nc, data = as.integer(0))
  } else {
    stop("unknown type")
  }
  return(z)
}

MatrixRbind = function(x, y) {
  if (is.null(x)) {
    if (is.null(y)) return(NULL)
    # we expect a deep copy from this function, so we make one
    return(MatrixSlice(y))
      
  } else if (is.null(y)) {
    return(MatrixSlice(x))
  }
  
  if (ncol(x) != ncol(y)) {
    stop("MatrixRbind : incompatible number of columns")
  }
  
  z = AllocMatrix(nrow(x) + nrow(y), ncol(x),
                  class(x)) 
  return(.Call("RbindMatrix", x, y, z, PACKAGE = kPhaseEmPkgName))
}

MatrixCbind = function(x, y) {
  # This function does not preserve the 'class' attribute
  if (is.null(x)) {
    if (is.null(y)) return(NULL)
    # we expect a deep copy from this function, so we make one
    return(MatrixSlice(y))
      
  } else if (is.null(y)) {
    return(MatrixSlice(x))
  }
  
  if (is.vector(x)) {
    if (is.vector(y)) {
      return(cbind(x, y))
    } else if (length(x) == nrow(y)) {
      return(cbind(matrix(x, ncol = 1), y))
    } else {
      return(cbind(matrix(x, nrow = 1), y))
    }
  } else {
    if (is.vector(y)) {
      if (length(y) == nrow(x)) {
        return(cbind(x, matrix(y, ncol = 1)))
      } else {
        return(cbind(x, matrix(y, nrow = 1)))
      }
    }
  }
  
  if (nrow(x) != nrow(y)) {
    stop("MatrixCbind : incompatible number of rows")
  }
  
  z = AllocMatrix(nrow(x), ncol(x) + ncol(y),
                  class(x))
  return(.Call("CbindMatrix", x, y, z, PACKAGE = kPhaseEmPkgName))
}

MatrixSlice = function(x, ri = NULL, ci = NULL, matType = NULL) {
  if (is.null(matType)) matType = class(x)
  if (is.null(ri)) ri = 1:nrow(x)
  if (is.null(ci)) ci = 1:ncol(x)

  ri = as.integer(ri - 1)
  ci = as.integer(ci - 1)
  r = AllocMatrix(length(ri), length(ci), matType)
  
  if (is.BinaryMatScaled(x) && is.BinaryMatScaled(r)) {
    .Call("ConvertMatrix", x, ri, ci, r, PACKAGE = kPhaseEmPkgName)
    
    r$rshift = x$rshift[ri + 1] 
    r$rmlt = x$rmlt[ri + 1]  
    r$cshift = x$cshift[ci + 1] 
    r$cmlt = x$cmlt[ci + 1]
  } else if (is.double(r)) {
    .Call("ConvertMatrixDbl", x, ri, ci, r, PACKAGE = kPhaseEmPkgName)
  } else {
    .Call("ConvertMatrix", x, ri, ci, r, PACKAGE = kPhaseEmPkgName)
  }
  return(r)
}

MatrixMaskedMult = function(x, y, ymask, matType = NULL) {
  return(.Call("MultMatMatMasked", x, y, ymask, PACKAGE = kPhaseEmPkgName))
}

MaskedPcaLeftRightUpdate = function(x, fm, msk = NULL, factInds = NULL,
                                    nIter = 1, tiedCols = TRUE) {
  # x = matrix of phased haplotypes
  # fm = FactoredMatrix
  # msk = mask matrix with 1's indicating which entries to use
  #       and 0's are those that should be treated as missing
  # factInds, nIter specify the components to update
  
  if (is.null(msk)) {
    msk = OnesMat(nrow(x), ncol(x))
  }
  if (is.null(factInds)) {
    factInds = rep(1:fm$nfact, each = nIter)
  }
  factInds0 = as.integer(factInds-1)
  
  fm$u = DeepCopyDbl(fm$u) 
  fm$v = DeepCopyDbl(fm$v)
  
  for (factIdx in factInds0) {
    if (factIdx < 0 || factIdx >= fm$nfact) {
      stop("invalid factor index")
    }
    .Call("MaskedPcaLRUpdate", x, msk, factIdx, fm, tiedCols,
          PACKAGE = kPhaseEmPkgName)      
  }
  return(fm)                              
} 


MaskedPcaGrad = function(x, fm, msk = NULL, tiedCols = TRUE) {
  # x = matrix of phased haplotypes
  # fm = FactoredMatrix
  # msk = mask matrix with 1's indicating which entries to use
  #       and 0's are those that should be treated as missing
  # factInds, nIter specify the components to update
  
  if (is.null(msk)) {
    msk = OnesMat(nrow(x), ncol(x))
  }
  fmGrad = FactoredMatrix(fm$nr, fm$nc, fm$nfact)
  .Call("MaskedPcaSubSpaceGrad", x, msk, fm, fmGrad,
        PACKAGE = kPhaseEmPkgName)      
  if (tiedCols) {
    idx1 = seq(from = 2, to = fm$nc, by = 2)
    idx2 = idx1 - 1
    vg = fmGrad$v[, idx1] + fmGrad$v[, idx2]
    fmGrad$v[, idx1] = vg
    fmGrad$v[, idx2] = vg
  }
  return(fmGrad)                              
}

MaskedPcaSqrs = function(x, fm, msk = NULL, tiedCols = TRUE) {
  # x = matrix of phased haplotypes
  # fm = FactoredMatrix
  # msk = mask matrix with 1's indicating which entries to use
  #       and 0's are those that should be treated as missing
  # factInds, nIter specify the components to update
  
  if (is.null(msk)) {
    msk = OnesMat(nrow(x), ncol(x))
  }
  fmGrad = FactoredMatrix(fm$nr, fm$nc, fm$nfact)
  .Call("MaskedPcaSubSpaceSqrs", x, msk, fm, fmGrad,
        PACKAGE = kPhaseEmPkgName)      
  if (tiedCols) {
    idx1 = seq(from = 2, to = fm$nc, by = 2)
    idx2 = idx1 - 1
    vg = fmGrad$v[, idx1] + fmGrad$v[, idx2]
    fmGrad$v[, idx1] = vg
    fmGrad$v[, idx2] = vg
  }
  return(fmGrad)                              
}

MaskedPcaSquaredError = function(x, fm, msk = NULL) {
  # calculates ||x - fm||^2  where the sum is over the 1's in 'msk'
  # or all entries if msk is NULL
  if (is.null(msk)) {
    msk = OnesMat(nrow(x), ncol(x))
  }
  r = .Call("MaskedPcaSqrErr", x, msk, fm, PACKAGE = kPhaseEmPkgName)       
  return(r)
}

RandFactoredMatrix = function(nr, nc, nfact) { 
  fm = FactoredMatrix(nr, nc, nfact)
  fm$u[] = rnorm(length(fm$u))
  fm$v[] = rnorm(length(fm$v))
  return(fm)
}

SubSpaceLearn = function(x, nfact = NULL, msk = NULL, fm = NULL,
                         tiedCols = FALSE, eps = 0.001,
                         alpha = 0.6, maxIter = 1000, keepTrace = FALSE,
                         epsInc = 1.1, epsDec = 0.5, verbose = TRUE) {
  nr = nrow(x)
  nc = ncol(x)

  if (is.null(fm)) {
    fm0 = RandFactoredMatrix(nr, nc, nfact)
  } else {
    nfact = fm$nfact
    fm0 = fm
  }

  eps0 = eps

  fm = fm0
  costSeq = c()
  epsSeq = c()

  if (keepTrace) {
    fmTrace = vector(mode = "list", length = maxIter)
  } else {
    fmTrace = NULL
  }

  t1 = proc.time()[[3]]

  for (j in 1:maxIter) {
    if (verbose) {
      print(sprintf("iteration %d", j))
    }
    if (keepTrace) {
      fmTrace[[j]] = fm
    }

    gr = MaskedPcaGrad(x, fm, msk = msk, tiedCols = tiedCols)
    if (alpha > 0) {
      sqrs = MaskedPcaSqrs(x, fm, msk = msk, tiedCols = tiedCols)
      gr$u = gr$u * (sqrs$u^-alpha)
      gr$v = gr$v * (sqrs$v^-alpha)
    }

    fmLast = fm

    fm$u = fm$u - gr$u * eps
    fm$v = fm$v - gr$v * eps
            
    if (tiedCols) {
      idx1 = seq(from = 2, to = fm$nc, by = 2)
      idx2 = idx1 - 1
      vAvg = (fm$v[, idx1] + fm$v[, idx2]) / 2
      fm$v[, idx1] = vAvg
      fm$v[, idx2] = vAvg
    }
  
    epsSeq = c(epsSeq, eps)
    costSeq = c(costSeq, MaskedPcaSquaredError(x, fm, msk = msk))

    if (length(costSeq) > 1) {
      n = length(costSeq)
      if (costSeq[n] > costSeq[n-1]) {
        eps = eps * epsDec
        fm = fmLast
      } else {
        eps = eps * epsInc
      }
    }
    for (ci in 1:nfact) {
      fm$u[, ci] = fm$u[, ci] / sqrt(sum(fm$u[, ci]^2))
    }
  }
  t2 = proc.time()[[3]]

  return(list(fm = fm, alpha = alpha, eps = eps, epsInc = epsInc, epsDec = epsDec,
              epsSeq = epsSeq, costSeq = costSeq,
              startTime = t1, endTime = t2))
}

MaskedPcaCoordOpt = function(x, nfact, msk = NULL, nIter = 10, verbose = TRUE) {
  if (is.null(msk)) {
    msk = OnesMat(nrow(x), ncol(x))
  }
  fm = RandFactoredMatrix(nrow(x), ncol(x), nfact)
  fm$u[, -1] = 0
  for (j in 1:nfact) {
    if (verbose) {
      print(sprintf("optimizing coordinate %d", j))
    }
    fm = MaskedPcaLeftRightUpdate(x, fm, msk = msk, factInds = rep(j, nIter))
    if (j + 1 <= nPCs) {
      fm$u[, j + 1] = rnorm(length(fm$u) / nfact)
    }
  }
  return(fm)
}

SubspaceLearn.Test = function(nr = 200, nc = 30, nfact = 2, eps = 0.01,
                             maxIter = 200) {
  # Tests the subspace learning algorithm in a case where we can check against
  # the SVD
  x = matrix(nrow = nr, ncol = nc, data = (runif(nr * nc) < 0.5))
  x0 = as.BinaryMat(x)
  
  svd0 = svd(x, nu = nfact, nv = nfact)
  svd0r = svd0$u %*% diag(svd0$d[1:nfact]) %*% t(svd0$v)
  svd0Cost = sum((x - svd0r)^2)
  
  SSL = SubSpaceLearn
  r0 = SSL(x0, nfact = nfact, alpha = 0, maxIter = maxIter, eps = eps)
  r1 = SSL(x0, nfact = nfact, alpha = 1, maxIter = maxIter, eps = eps)
  r06 = SSL(x0, nfact = nfact, alpha = 0.6, maxIter = maxIter, eps = eps)
  
  e0 = r0$costSeq / svd0Cost - 1
  e1 = r1$costSeq / svd0Cost - 1
  e06 = r06$costSeq / svd0Cost - 1
  
  plot(e0, ylim = range(e0, e1, e06), type = "l", log = "y",
       main = "1 - (COST / SVD-COST)")
  lines(e1, col = "red")
  lines(e06, col = "blue")
  
  legend(x = "topright", col = c("black", "red", "blue"),
         lty = c("solid", "solid", "solid"),
         legend = c("grad asc", "newton", "accel = 0.6"))
  return(list(r0 = r0, r06 = r06, r1 = r1, x = x))
}


is.ByteMat = function(obj) {
  return(class(obj) == "BinaryMat" && obj$type == "ByteMat")
}

MatrixToIntMat = function(x) {
  nr = nrow(x)
  nc = ncol(x)
  r = matrix(nrow = nr, ncol = nc, data = as.integer(0))
  return(.Call("ConvertMatrix", x, 0:(nr-1), 0:(nc-1), r,
               PACKAGE = kPhaseEmPkgName))
}

MatrixToDblMat = function(x) {
  nr = nrow(x)
  nc = ncol(x)
  r = matrix(nrow = nr, ncol = nc, data = as.double(0))
  return(.Call("ConvertMatrixDbl", x, 0:(nr-1), 0:(nc-1), r,
               PACKAGE = kPhaseEmPkgName))
}

is.BinaryMat = function(obj) {
  return(class(obj) == "BinaryMat" && obj$type == "BinaryMat")
}

MatrixSetCol = function(x, vals, idx, makeCopy = TRUE) {
	if (makeCopy) {
    x = MatrixSlice(x)
  }
  if (length(vals) == 1) vals = rep(vals, nrow(x))
  ri = 0:(nrow(x)-1)
  vals = matrix(vals, ncol = 1)		              
  return(.Call("SetMatrixSlice", vals, ri, idx-1, x, PACKAGE = kPhaseEmPkgName))
}

MatrixSetRow = function(x, vals, idx, makeCopy = TRUE) {
	if (makeCopy) {
    x = MatrixSlice(x)
  }          
  if (length(vals) == 1) vals = rep(vals, ncol(x))
  ci = 0:(ncol(x)-1)
  vals = matrix(vals, nrow = 1)		              
  return(.Call("SetMatrixSlice", vals, idx-1, ci, x, PACKAGE = kPhaseEmPkgName))
} 

MatrixSetSlice = function(x, sliceVals, rowInds = NULL, colInds = NULL,
                          makeCopy = TRUE) {
	if (makeCopy) {
    x = MatrixSlice(x)
  }
  if (length(dim(sliceVals)) == 2) {
    if (is.null(rowInds)) rowInds = 1:nrow(sliceVals)
    if (is.null(colInds)) colInds = 1:ncol(sliceVals)
  } else {
    if (is.null(rowInds)) rowInds = 1:nrow(x)
    if (is.null(colInds)) colInds = 1:ncol(x)
    sliceVals = matrix(rep(sliceVals, length(rowInds) * length(colInds)),
                       nrow = length(rowInds), ncol = length(colInds))
  }
           
	ri = as.integer(rowInds-1)
	ci = as.integer(colInds-1)           
  return(.Call("SetMatrixSlice", sliceVals, ri, ci, x,
         PACKAGE = kPhaseEmPkgName))
}

##################################################
# S3 wrappers from these functions
##################################################

# use these to take slices of rows and columns
subset.BinaryMat = function(x, subset, ...) {
  return(MatrixSlice(x, ri = subset, ...))
}

subset.BinaryMat = function(x, subset, select, drop = FALSE, ...) {
  return(MatrixSlice(x, ri = subset, ci = select, ...))
}

subset.BinaryMatScaled = function(x, subset, ...) {
  r = MatrixSlice(x, ri = subset, ci = select, ...)
  if (is.BinaryMatScaled(r) && is.BinaryMatScaled(x)) {
    r$rshift = x$rshift[subset] 
    r$rmlt = x$rmlt[subset]
  }
  return(r)
}

subset.BinaryMatScaled = function(x, subset, select, drop = FALSE, ...) {
  r = MatrixSlice(x, ri = subset, ci = select, ...)
  if (is.BinaryMatScaled(r) && is.BinaryMatScaled(x)) {
    if (!is.null(subset)) {
      r$rshift = x$rshift[subset] 
      r$rmlt = x$rmlt[subset]
    }   
    if (!is.null(select)) {
      r$cshift = x$cshift[select] 
      r$cmlt = x$cmlt[select]
    }
  }
  return(r)
}

SetSlice.BinaryMat = function(x, ...) {
  return(MatrixSetSlice(x, ...))
}

Hash.BinaryMat = function(x) {
  return(MatrixHash(x))
}

dim.OnesMat = function(x) {
  return(c(x$nr, x$nc))
}

dim.BinaryMat = function(x) {
  return(c(x$nr, x$nc))
}

dim.ByteMat = function(x) {
  return(c(x$nr, x$nc))
}  

dim.BinaryMatScaled = function(x) {
  return(c(x$nr, x$nc))
} 

dim.FactoredMatrix = function(x) {
  return(c(x$nr, x$nc))
}

cbind.BinaryMat = function(x, y) {
  return(MatrixCbind(x, y))
} 

cbind.ByteMat = function(x, y) {
  return(MatrixCbind(x, y))
}

rbind.BinaryMat = function(x, y) {
  return(MatrixRbind(x, y))
} 

rbind.ByteMat = function(x, y) {
  return(MatrixRbind(x, y))
}

as.integer.BinaryMat = function(x) {
  return(MatrixToIntMat(x))
}  

as.double.BinaryMatScaled = function(x) {
  return(MatrixToDblMat(x))
} 

as.integer.BinaryMatScaled = function(x) {
  # WARNING : ignores shift and scaling factors
  return(MatrixToIntMat(x))
}

as.ByteMat = function(x, makeCopy = TRUE) {
  nr = nrow(x)
  nc = ncol(x)
  r = ByteMat(nr, nc)
  return(.Call("ConvertMatrix", x, 0:(nr-1), 0:(nc-1), r,
               PACKAGE = kPhaseEmPkgName))
}

as.BinaryMat = function(x, makeCopy = TRUE) {
  nr = nrow(x)
  nc = ncol(x)
  r = BinaryMat(nr, nc)
  return(.Call("ConvertMatrix", x,  0:(nr-1), 0:(nc-1), r,
               PACKAGE = kPhaseEmPkgName))
}

SetCol.BinaryMat = function(x, vals, idx, makeCopy = TRUE) {
	return(MatrixSetCol(x, vals, idx, makeCopy))
}

SetRow.BinaryMat = function(x, vals, idx, makeCopy = TRUE) {
	return(MatrixSetRow(x, vals, idx, makeCopy))
}

"[.BinaryMat" = function(x, ri = NULL, ci = NULL) {
  return(subset(x, subset = ri, select = ci))
}

"[<-.BinaryMat" = function(x, ri = NULL, ci = NULL, value) {
  MatrixSetSlice(x, sliceVals = value, rowInds = ri, colInds = ci,
                 makeCopy = FALSE)
} 

"[.BinaryMatScaled" = function(x, ri = NULL, ci = NULL) {
  return(subset(x, subset = ri, select = ci))
}

"[<-.BinaryMatScaled" = function(x, ri = NULL, ci = NULL, value) {
  MatrixSetSlice(x, sliceVals = value, rowInds = ri, colInds = ci,
                 makeCopy = FALSE)
}

t.BinaryMat = function(x) {                  
  r = BinaryMat(ncol(x), nrow(x))
  return(.Call("TransposeMatrix", x, r))
} 

t.ByteMat = function(x) {                  
  r = ByteMat(ncol(x), nrow(x))
  return(.Call("TransposeMatrix", x, r))
}

rowSums.BinaryMat = function(x) {
  return(MatrixRowSums)
}
  
colSums.BinaryMat = function(x) {
  return(MatrixColSums)
} 

rowSums.ByteMat = function(x) {
  return(MatrixRowSums)
}
  
colSums.ByteMat = function(x) {
  return(MatrixColSums)
}


##################################################
# S3 wrappers from these functions
##################################################


BinaryMatTests = function() {
  BinaryMatSliceTest()
  BinaryMatCbindRbindTest()
}

BinaryMatSliceTest = function() {
  print("SliceTest")
  x0 = matrix(nrow = 4, ncol = 6,
              data = (runif(24) < 0.5))
  x1 = matrix(nrow = 2, ncol = 6,
              data = (runif(12) < 0.5))
  x2 = x0
  x2[2:3, ] = x1

  for (j in 1:2) {
    if (j == 1) castFunc = as.BinaryMat
    if (j == 2) castFunc = as.ByteMat
    x0bm = castFunc(x0)
    x1bm = castFunc(x1)

    x2bm = x0bm

    x2bm[2:3, ] = x1bm
    nDiff1 = sum(x2 != as.integer(x2bm))
    x2bm[2:3, 1:ncol(x2bm)] = x1bm
    nDiff2 = sum(x2 != as.integer(x2bm))

    if (nDiff1 > 0 || nDiff2 > 0) {
      stop("Test FAILED")
    }
  }
  
  x0bm = as.BinaryMat(x0)
  v0 = (runif(nrow(x0bm)) < 0.5)
  x0bm[, 3] = v0
  x0[, 3] = v0
  
  if (sum(as.integer(x0bm) != x0) > 0) {
    stop("Test Failed")
  }
  
  print("Test PASSED")
  return(TRUE)
}


BinaryMatCbindRbindTest = function() {
  print("CbindRbindTest")
  x0 = matrix(nrow = 3, ncol = 3,
              data = (runif(9) < 0.5))
  x1 = matrix(nrow = 3, ncol = 3,
              data = (runif(9) < 0.5))
  x2 = cbind(x0, x1)
  x3 = rbind(x0, x1)

  for (j in 1:2) {
    if (j == 1) castFunc = as.BinaryMat
    if (j == 2) castFunc = as.ByteMat
    x0bm = castFunc(x0)
    x1bm = castFunc(x1)

    x2bm = cbind(x0bm, x1bm)
    x2bm2 = cbind(x0bm, x1)

    x3bm = rbind(x0bm, x1bm)
    x3bm2 = rbind(x0bm, x1)

    x3bm3 = x3bm[1:2, 1:2]

    # check that nrow and ncol are being overridden
    if (nrow(x3bm) != nrow(x3) || ncol(x3bm) != ncol(x3)
        || nrow(x2bm) != nrow(x2) || ncol(x2bm) != ncol(x2)) {
      stop("test FAILED")
    }

    nDiff1 = sum(x2 != as.integer(x2bm))
    nDiff12 = sum(x2 != as.integer(x2bm2))
    nDiff2 = sum(x3 != as.integer(x3bm))
    nDiff22 = sum(x3 != as.integer(x3bm2))

    nDiff33 = sum(x3[1:2, 1:2] != as.integer(x3bm3))

    if (nDiff1 > 0 || nDiff2 > 0 || nDiff12 > 0 || nDiff22 > 0) {
      stop("Test FAILED")
    }
  }
  print("Test PASSED")
  return(TRUE)
}

