# Generate a random matrix of binary alleles for testing the functions
RndAlleleDb = function(nSamp, nSnp) {
  x = as.integer(2*runif (nSnp*nSamp*2))
  return( matrix(ncol=nSnp, data=x) )
}

AlleleDbFromGtype = function(gtypeMat) {
  m0 = matrix(nrow=(2*nrow(gtypeMat)), ncol=ncol(gtypeMat),
              data=as.integer(2))
  i = seq(from=2, to=nrow(m0), by=2)
  m0[i,][gtypeMat == 0] = as.integer(0)
  m0[i-1,][gtypeMat == 0] = as.integer(0)
  
  m0[i,][gtypeMat == 1] = as.integer(0)
  m0[i-1,][gtypeMat == 1] = as.integer(1)
  
  m0[i,][gtypeMat == 2] = as.integer(1)
  m0[i-1,][gtypeMat == 2] = as.integer(1)
  
  return(m0)
}

plot.phasing = function(z, method="average", main="", clust.snp.start=NULL,
                         x=0:nrow(z), y=0:ncol(z)) {
	if (method == "none") {
		image(z, main=main, x=x, y=y)

		for (j in 0:nrow(z)) {
			if (j %% 2 == 0) abline(v=x[j], lty="solid")
			#else abline(v=j, lty="dotted")
		}
	} else {
	  if (ncol(z) > 100) {
	    if (is.null(clust.snp.start)) {
	      i1 = 1:40 + as.integer(ncol(z)/2)
        i1 = i1[i1 < ncol(z)]
      } else {       
	      i1 = 1:40 + clust.snp.start
        i1 = i1[i1 < ncol(z)]
      }
	    dist.obj = dist(z[,i1])
    } else {
		  dist.obj = dist(z)
	  }
		hclust.obj = hclust(dist.obj, method=method)
		image(z[hclust.obj$order,], main=main, x=x, y=y)
	  return( hclust.obj$order )
	}
}

RandPhasing = function(x) {
  # x is a 2M by N matrix
  # For each i=1,...,M, and j=1,..,N
  #   we randomly swap  x[2i,j] and x[2i-1,j]
  # If rows 2i and 2i-1 are two haplotypes
  # belonging to one diploid individual, this
  # function results in a random phasing
	n.samp = nrow(x)/2
	n.snp = ncol(x)

	s1 = 1:n.samp

	for (j in 1:n.samp) {
		i1 = 2*j-1
		i2 = i1 + 1
		
		i3 = runif (n.snp) < 0.5
		z = x[i1,i3]      
		x[i1,i3] = x[i2,i3]
		x[i2,i3] = z
	}
	return(x)
}

switch.checks = function(phase.known.tbl, n.range=1) {
	m1 = phase.known.tbl

	n.samp = nrow(phase.known.tbl)/2
	n.snp = ncol(phase.known.tbl)

	ret1 = matrix(nrow=0, ncol=3)

	s1 = 1:n.samp

	if (n.range == 1) {
		for (j in s1) {
			i1 = 2*j-1

			i3 = which(phase.known.tbl[i1,])

			if (length(i3) > 1) {
				snp1 = i3[-1]
				snp2 = i3[-length(i3)]
				s2 = rep(j, length(snp1))
				m2 = matrix(nrow=length(s2), ncol=3)
				m2[,1] = s2
				m2[,2] = snp1
				m2[,3] = snp2

				ret1 = rbind(ret1, m2)
			}
		}
	} else {
		for (j in s1) {
			i1 = 2*j-1

			i3 = which(phase.known.tbl[i1,])

			if (length(i3) > 1) {
				snp1 = i3[-1]
				snp2 = i3[-length(i3)]
				s2 = rep(j, length(snp1))
				m2 = matrix(nrow=0, ncol=3)
				
				for (k2 in 2:length(i3)) {
					ll.lim = max(1, k2-n.range)
					for (j2 in ll.lim:(k2-1)) {
						m2 = rbind(m2, c(j, i3[j2], i3[k2]) )
					}
				}

				ret1 = rbind(ret1, m2)
			}
		}
	}
	
	return(ret1 )
}

calc.switch.err.v2 = function(phase.test, trio.phased, phasing.known, snp.rng) {
  x = trio.phased
  y = phase.test
  ret = .Call("ComputePhasingErrs", x, y, phasing.known,
                       nrow(x)/2, ncol(x), 
                       snp.rng[1]-1, snp.rng[2]-1, 
                       ncol(x), T)
  return(ret)
}
