#include "phase_em.hpp"

namespace PhaseEM {

class PhaseEmTester1 : public PhaseEmInference {
public:
	PhaseEmTester1(PhaseEmModel& p);

	virtual void BackwardProcess(int cur_allele, int snp_idx,
			                         const RealVec& bkwd_msg);

	const RealVec& log_liks() const { return(log_liks_); }

public:
	RealVec log_liks_;
};

PhaseEmTester1::PhaseEmTester1(PhaseEmModel& p) : PhaseEmInference(p) {
	log_liks_ = RealVec(params_.n_snp(), 0.0);
}

void PhaseEmTester1::BackwardProcess(int cur_allele, int snp_idx,
	                                   const RealVec& bkwd_msg) {
	const RealVec& fwd_msg = this->ForwardMsg(snp_idx - 1);
  log_liks_[snp_idx] = log(DotProd(bkwd_msg, fwd_msg));
}

}  // namespace PhaseEM {



extern "C" {

SEXP PhaseEmLikTest1_RI(SEXP params_sxp, SEXP cases_sxp, SEXP case_idx_sxp,
				                SEXP pliks_sxp) {
	using namespace PhaseEM;
	using namespace Util;

	PhaseEmModel * mdl = PhaseEmModelFactory(params_sxp, cases_sxp);
	PointerDeleter<PhaseEmModel> mdl_deleter(mdl);
	PhaseEmTester1 inf_obj(*mdl);
	inf_obj.set_rescale_freq(mdl->n_snp() + 10);

	double * pliks = (pliks_sxp == R_NilValue) ? NULL : REAL(pliks_sxp);
	if (pliks && LENGTH(pliks_sxp) != mdl->n_snp()) {
		error("PhaseEMParamUpd : input buffers wrong length\n");
	}

	int case_idx = GetInt(case_idx_sxp, 0, NULL);
	inf_obj.set_case_index(case_idx);
	double cur_plik = inf_obj.ForwardAlgo();
	inf_obj.BackwardAlgo();

	for (int i = 0; i < mdl->n_snp(); ++i) {
		pliks[i] = inf_obj.log_liks()[i];
	}
	return R_NilValue;
}

}  // extern "C"
