#include "switch_error.hpp"


extern "C" {
	SEXP SwitchErrorCalc(SEXP true_ph_sxp, SEXP ph_known_sxp, SEXP test_ph_sxp,
			                 SEXP n_errs_sxp, SEXP n_tests_sxp) {
  	using namespace SwitchError;
  	using namespace BinaryMatrix;
  	using namespace Util;

  	PointerDeleter<CastableMatI> true_ph(OwningCastableMatFactory(true_ph_sxp));
  	PointerDeleter<CastableMatI> ph_known(OwningCastableMatFactory(ph_known_sxp));
  	PointerDeleter<CastableMatI> test_ph(OwningCastableMatFactory(test_ph_sxp));

    SwitchErrorCalculator::Calc(*true_ph.get(), *ph_known.get(), *test_ph.get(),
    		                        INTEGER(n_errs_sxp), INTEGER(n_tests_sxp));
  	return(R_NilValue);
	}


  SEXP SwitchErrByDist(SEXP true_ph_sxp, SEXP ph_known_sxp, SEXP test_ph_sxp,
  		                 SEXP posns_sxp, SEXP bins_sxp,
  		                 SEXP n_errs_sxp, SEXP n_tests_sxp) {
  	using namespace SwitchError;
  	using namespace BinaryMatrix;
  	using namespace Util;

  	PointerDeleter<CastableMatI> true_ph(OwningCastableMatFactory(true_ph_sxp));
  	PointerDeleter<CastableMatI> ph_known(OwningCastableMatFactory(ph_known_sxp));
  	PointerDeleter<CastableMatI> test_ph(OwningCastableMatFactory(test_ph_sxp));

    RealVec posns, bins;
    SxpToDblVec(posns_sxp, &posns);
    SxpToDblVec(bins_sxp, &bins);
    int n = bins.size();

    IntVec n_errs(n, 0), n_tests(n, 0);

  	SwErrByDist::Calc(*true_ph.get(), *ph_known.get(), *test_ph.get(),
  			              posns, bins, &n_errs, &n_tests);

  	for (int i = 0; i < n; ++i) {
  		INTEGER(n_errs_sxp)[i] = n_errs[i];
  		INTEGER(n_tests_sxp)[i] = n_tests[i];
  	}
  	return(R_NilValue);
  }
}  // extern "C" {
