#ifndef ARADMX_PHASED_HPP_
#define ARADMX_PHASED_HPP_

#include "util.hpp"
#include "aradmx_fwd_bkwd.hpp"
#include "aradmx.hpp"


//#define EXACT_INF_DEBUG_CHECKS 1
namespace Aradmx {
namespace Phased {

using namespace Util;
using namespace Aradmx::FwdBkwd;;

class PhasedModelParams : public ModelParams {
public:
	struct State {
		unsigned char anc_;
		unsigned char jump_idx_;
	};

	typedef vector<State> StateMat;

	PhasedModelParams() : ModelParams() {
	  state_size_ = 0;
	  hap_ = NULL;
	}

	void set_hap(Hap& h) {
		hap_ = h;
	}

	void Fill(SEXP drop_lookups_sxp, SEXP max_drop_sxp,
	          SEXP state_mat_sxp, SEXP win_sz_sxp,              // The state matrix
	          SEXP L2RMap, SEXP R2LMap,     // mappings to absorb messages
	          SEXP anc_transprb_sxp, SEXP anc_priors_sxp, // Recombination and ancestral prior parameters
	          SEXP bin_trees_sxp);

	void FirstSnpZero(double * r);
	void GetAncPrbs(int snp_idx, const double * cur_pttl, double * anc_prbs) const;
	void CondProbs(const IntVec& pop_lab, int snp_idx, double * r);
	void ObservePhasing(int snp_idx, int phase_or, double * r) const {}

public:
	StateMat state_mat_;
  int state_size_;

  Hap hap_;
};

}  // namespace Aradmx {
}  // namespace Phased {


extern "C" {

	SEXP AradmxPhasedExactFwdBkwd_RI(SEXP hap_sxp, SEXP drop_lookups_sxp, SEXP max_drop_sxp,
	                              SEXP junc_tree_sxp, SEXP win_sz_sxp,              // The state matrix
	                              SEXP anc_transprb_sxp, SEXP anc_priors_sxp, // Recombination and ancestral prior parameters
	                              SEXP bin_trees_sxp,    // Parent sets and their observed counts/multinomial parameters
	                              SEXP pop_lab_sxp,
	                              SEXP checkpointed_sxp,
	                              SEXP marg_lik_sxp, SEXP anc_prbs_sxp);

}  // extern "C"

#endif /*ARADMX_PHASED_HPP_*/
