kAncIndexOne = kModelIndexOne = as.integer(1)

DebugMsg = function(...) {
  return(NULL)
  msg = sprintf(...)
  print(msg)
  write(msg, file = "C:\\Users\\n\\Documents\\Rpackages\\PhaseEmEclipseProject\\debug.txt",
        append = TRUE)
}

AradmxPackDropLookups = function(dropLookups) {
  DebugMsg("entering AradmxPackDropLookups")
  # dropLookups is a vector of type list with nPop elements
  # dropLookups[[popIdx]] is a matrix of dimension [window-size, num-markers]
  # and the (i, j)'th element is the applicable parent set when there was an
  # ancestry change between markers (i-j-1) and (i-j).  If j = 0, there is
  # guaranteed to be an ancestry change and the parent set should have all
  # edges clipped.
  
  nPop = length(dropLookups)
  maxDrop = dim(dropLookups[[1]])[1]
  dl = array(as.integer(0), c(nPop, dim(dropLookups[[1]])))
  for (popIdx in 1:nPop) {
    dl[popIdx,,] = dropLookups[[popIdx]]
  }
  
  DebugMsg("dim(dl) = %d (n-pop), %d, %d", dim(dl)[1], dim(dl)[2], dim(dl)[3])
  
  dropLookupsPacked = .Call("AradmxPackDropLookups_RI", dl,
                            PACKAGE = kPhaseEmPkgName)
  DebugMsg("exiting AradmxPackDropLookups")
  return(list(maxDrop = maxDrop, dropLookupsPacked = dropLookupsPacked))
}

AradmxBuildDropLookups = function(binTrees, p) { 
  DebugMsg("entering AradmxBuildDropLookups")
  ## binTrees is a vector of length R where index [[1]] is of higher model
  ## order than index [[2]] which is of higher order than index [[3]]
  ## ... until index [[R]] which is a 0'th order model -- a model with no
  ## splits and gives marginal allele frequencies but conditions on no
  ## neighboring splits.
  ##
  ## In the model, ancestral breaks clip edges and assuming there is a break
  ## between markers (k-j+1) and (k-j) we must decide which model we can use
  ## to calculate the probability of the observed allele at marker index 'k'
  ##
  ## We return an M-by-N matrix X.  The element X[j, k] contains the index
  ## of the tree with no clipped edges assuming that there was an ancestral
  ## break between SNP's   (k-j+1) and (k-j) and no further breaks before
  ## the k'th SNP.
  
  maxDrop = p$modelOrd + 1      
  DebugMsg("maxDrop = %d, length(binTrees[[1]]) = %d",
           maxDrop, length(binTrees[[1]]))
  r = .Call("FirstUnclippedTreeIndex_RI", binTrees, maxDrop,
            PACKAGE = kPhaseEmPkgName)
  r = matrix(r, nrow = maxDrop)
  DebugMsg("exiting AradmxBuildDropLookups")
  return(r)
}

AradmxOrderCrossvalLLiks = function(p, ancDb, binTrees, nFolds, die = 0) {
  DebugMsg("entering AradmxOrderCrossvalLLiks")
  nOrds = length(binTrees)
  nMarkers = ncol(ancDb)
  
  r = matrix(nrow = nMarkers, ncol = nOrds, data = 0.0) 
  r0 = rep(0.0, nMarkers)
  
  if (die == 1) stop("die: 1")                        
  for (foldIdx in 1:nFolds) {
    trInds = sample(1:nrow(ancDb), as.integer(nrow(ancDb) * 0.9))
    tstInds = (1:nrow(ancDb))[-trInds]
    xTr = as.BinaryMat(t(ancDb[trInds,, drop = FALSE ]))
    xTst = as.BinaryMat(t(ancDb[tstInds,, drop = FALSE ]))
    trWts = rep(1.0, length(trInds))
    tstWts = rep(1.0, length(tstInds))
    if (die == 2) stop("die: 2")
    
    for (ordIdx in 1:nOrds) {
      if (die == 3) stop("die: 3")  
      .Call("IncLeafCounts_RI", xTr, trWts, TRUE, p, binTrees[[ordIdx]],
		        PACKAGE = kPhaseEmPkgName)
      .Call("PerMarkerLLiks_RI", xTst, tstWts, binTrees[[ordIdx]],
		        r0, PACKAGE = kPhaseEmPkgName)
      r[, ordIdx] = r[, ordIdx] + r0
    }
  } 
  DebugMsg("exiting AradmxOrderCrossvalLLiks")
  return(r)
}

AradmxCrossvalModelOrds = function(ancDbs, binTrees, p) { 
  DebugMsg("entering AradmxCrossvalModelOrds")
  nPop = length(ancDbs)
  cvLogLiks = vector(mode = "list", length = nPop)
  cvLLikMat = NULL
  for (popIdx in 1:nPop) {
    cvLogLiks[[popIdx]] = AradmxOrderCrossvalLLiks(p, ancDbs[[popIdx]],
                                                   binTrees[[popIdx]], 10)
    if (is.null(cvLLikMat)) {
      cvLLikMat = cvLogLiks[[popIdx]]
    } else {
      cvLLikMat = cvLLikMat + cvLogLiks[[popIdx]]
    }
  }
  maxLLiks = rep(-Inf, nrow(cvLLikMat))
  bestOrds = rep(-1, nrow(cvLLikMat))
  for (j in 1:ncol(cvLLikMat)) {
    i = (cvLLikMat[, j] > maxLLiks)
    maxLLiks[i] = cvLLikMat[i, j]
    bestOrds[i] = j
  }                        
  DebugMsg("exiting AradmxCrossvalModelOrds")
  return(list(cvLogLiks = cvLogLiks, cvLLikMat = cvLLikMat,
              bestOrds = bestOrds, maxLLiks = maxLLiks))
}

AradmxResetLeafCounts = function(ancDbs, binTrees, p) {
  DebugMsg("entering AradmxResetLeafCounts")
  zeroCounts = TRUE
  nPop = length(ancDbs)
  
  for (popIdx in 1:nPop) {
    x = t(ancDbs[[popIdx]])
    wts = rep(1.0, ncol(x))
    bt = binTrees[[popIdx]]
    
    for (j in 1:length(bt)) {
      .Call("IncLeafCounts_RI", as.BinaryMat(x), wts, zeroCounts, p, bt[[j]],
            PACKAGE = kPhaseEmPkgName)
    }
    binTrees[[popIdx]] = bt
  }              
  DebugMsg("exiting AradmxResetLeafCounts")
  return(binTrees)
}

BiDirectionalAradmxModel = function(ancDbs, controlObj = NULL) {
  if (is.null(controlObj)) {
    controlObj = AradmxControl()
  }
  
  revControl = fwdControl = controlObj
  fwdControl$revSnpOrd = FALSE        
  revControl$revSnpOrd = TRUE
  fwdMdl = AradmxModel(ancDbs, controlObj = fwdControl)
  revMdl = AradmxModel(ancDbs, controlObj = revControl)
  return(list(fwdMdl = fwdMdl, revMdl = revMdl, isBiDirectional = TRUE))
}

AradmxModel = function(ancDbs, controlObj = NULL) {
  if (is.null(controlObj)) {
    controlObj = AradmxControl()
  }
  p = controlObj
  
  nPop = length(ancDbs)
  nSnp = ncol(ancDbs[[1]])
  ancDbs0 = ancDbs
  
  if (controlObj$bootStrap) {
    ancDbs = BootstrapAncDbs(ancDbs)
  }
    
  ancDbs = AllPositiveCoding(ancDbs)
  ancDbs0 = AllPositiveCoding(ancDbs0)
  
  if (p$revSnpOrd) { 
    ancDbs0 = ReverseSnpOrd(ancDbs0)
    ancDbs = ReverseSnpOrd(ancDbs)
  }

  phJt = AradmxPhasedStateMat(nPop, p$modelOrd)
  unphJt = AradmxBnUnphStateMat(nPop, p$modelOrd + 1, reduced = FALSE)
  unphJtRed = AradmxBnUnphStateMat(nPop, p$modelOrd + 1, reduced = TRUE)

  popLabels = c()

  allDb = NULL
  if (p$tiedStruct) {
    allDb = NULL
    for (popIdx in 1:nPop) {
      popLabels = c(popLabels, rep(popIdx, nrow(ancDbs[[popIdx]])))
      allDb = rbind(allDb, ancDbs[[popIdx]])
    }
  }
  
  binTrees = vector(mode = "list", length = nPop)
  dropLookups = vector(mode = "list", length = nPop)

  for (popIdx in 1:nPop) {
    if (p$tiedStruct) {
      x = allDb
    } else {
      x = ancDbs[[popIdx]]
      popLabels = rep(popIdx, nrow(ancDbs[[popIdx]]))
    }
    
    binTrees[[popIdx]] = AradmxBuildTrees(x, popIdx, controlObj, popLabels = popLabels)
  }
  if (controlObj$crossValidate) {
    crossValStats = AradmxCrossvalModelOrds(ancDbs, binTrees, controlObj)
  } else {
    crossValStats = NULL
  }

  for (popIdx in 1:nPop) {
    dropLookups[[popIdx]] = AradmxBuildDropLookups(binTrees[[popIdx]], controlObj)
  }
  binTrees = AradmxResetLeafCounts(ancDbs0, binTrees, p)     

  r = AradmxPackDropLookups(dropLookups)
  maxDrop = r$maxDrop
  dropLookupsPacked = r$dropLookupsPacked
  
  if (maxDrop != p$modelOrd + 1) stop("z")
  
  if (controlObj$keepForDebugging) {
    dropLookupsUnpacked = dropLookups
  } else {
    dropLookupsUnpacked = NULL
  }
  
  mdl = list(dropLookups = dropLookupsPacked,
             dropLookupsUnpacked = dropLookupsUnpacked,
             maxDrop = maxDrop,
             binTrees = binTrees,
             controlObj = controlObj,
             nPop = nPop, nSnp = nSnp,
             unphJt = unphJt,
             unphJtRed = unphJtRed,
             phJt = phJt,
             crossValStats = crossValStats,
             isBiDirectional = FALSE)
  mdl = AradmxMakePackedTrees(mdl)
  return(mdl)
}

AradmxMakePackedTrees = function(mdl) {
  DebugMsg("entering AradmxMakePackedTrees")
  
  if (mdl$controlObj$addCanaries) stop("canaries not implemented")  
  binTrees = mdl$binTrees
  
  packCounts = initZeroes = FALSE
  
  treePosnsDim = c(length(binTrees[[kAncIndexOne]]), mdl$nPop, mdl$nSnp)
  treePosns = array(as.integer(-1), treePosnsDim)
                           
  DebugMsg("entering AradmxPackInitTrees_RI")
  binTreesPacked = .Call("AradmxPackInitTrees_RI", binTrees, treePosns,
		                      packCounts, initZeroes, mdl$controlObj$addCanaries,
                          PACKAGE = kPhaseEmPkgName) 
  
  binTreesPacked = list(posns = treePosns, numClipTrees = length(binTrees[[1]]),
                        nSnp = mdl$nSnp, nPop = mdl$nPop,
                        trees = binTreesPacked)
                        
  mdl$binTreesPacked = binTreesPacked 
  
  DebugMsg("exiting AradmxMakePackedTrees")
  return(mdl)
}

GetJumpProbs = function(nSnp, jumpProb = NULL, admixTime = NULL,
                        snpDists = NULL, revSnpOrd = FALSE) { 
  DebugMsg("entering GetJumpProbs")
  if (is.null(snpDists)) DebugMsg("snpDists = NULL")
  if (is.null(jumpProb)) DebugMsg("jumpProb = NULL")
  
  if (is.null(snpDists) || is.null(admixTime)) {
    if (is.null(admixTime)) {
      noJump = 1 - jumpProb
    } else {
      noJump = exp(-admixTime)
    }
    noJump = rep(noJump, nSnp)
    r = rbind(noJump, 1-noJump)
  } else {
    if (revSnpOrd) {
      snpDists = c(0, rev(snpDists[-1]) )
    }
  
    noJump = exp(-snpDists * admixTime)
    noJump[1] = 1.0
    r = rbind(noJump, 1-noJump)
  } 
  
  DebugMsg("exiting GetJumpProbs")
  return(r)
}

     
AradmxPhasedVitRet = function(mdl, vitTrace, statMat) {
  vitTrace = vitTrace + as.integer(1)
  if (mdl$controlObj$revSnpOrd) {
    vitTrace = rev(vitTrace)
  }
  
  ancSeq = stateMat[2, vitTrace] + as.integer(1)
  jumpSeq = (stateMat[1, vitTrace] == 1)

  return(list(ancSeq = ancSeq, jumpSeq = jumpSeq, vitTrace = vitTrace))
}
      
AradmxUnphasedVitRet = function(mdl, vitTrace, statMat) {
  vitTrace = vitTrace + as.integer(1)  
  if (mdl$controlObj$revSnpOrd) {
    vitTrace = rev(vitTrace)
  }
  
  anc1 = stateMat[4, vitTrace] + as.integer(1)
  anc2 = stateMat[6, vitTrace] + as.integer(1)
  
  nPop = mdl$nPop
  ancCounts = matrix(nrow = nPop, ncol = nSnp, data = 0)
  for (k in 1:nPop) {
    ancCounts[k, ] = (anc1 == k) + (anc2 == k)
  }
  
  allelePhase = stateMat[3, vitTrace]

  return(list(allelePhase = allelePhase, ancCounts = ancCounts,
              anc1 = anc1, anc2 = anc2))
}

  
AradmxPhasedMargRet = function(mdl, margLik, ancPrbs, expCounts, stateMat,
                                 moreExpectedCounts) {
  if (!is.null(expCounts) && sum(is.na(expCounts)) > 0) {
    print("NA's found in output")
  }

  if (moreExpectedCounts) {
    v1 = rep(0.0, ncol(stateMat))
    v1[(stateMat[1, ] == 1)] = 1.0     
    jmpPrbs = v1 %*% expCounts
  } else {
    jmpPrbs = NULL
  }

  if (mdl$controlObj$revSnpOrd) {
    i = rev(1:(mdl$nSnp))
    ancPrbs = ancPrbs[, i]
  }

  return(list(margLik = margLik, ancPrbs = ancPrbs, jmpPrbs = jmpPrbs))
}
      
AradmxUnphasedMargRet = function(mdl, margLik, ancPrbs, expCounts, stateMat,
                                 moreExpectedCounts) {
  if (!is.null(expCounts) && sum(is.na(expCounts)) > 0) {
    stop("NA's found in output")
  } 
  if (moreExpectedCounts) {
    nStt = ncol(stateMat)
    v1 = rep(0.0, nStt)
    v1[(stateMat[6, ] == 1) & (stateMat[3, ] == 1)] = 1.0     
    jmpPrbsGrp1 = v1 %*% expCounts
    
    v1 = rep(0.0, nStt)
    v1[(stateMat[7, ] == 1) & (stateMat[3, ] == 1)] = 1.0     
    jmpPrbsGrp2 = v1 %*% expCounts
    
    v1 = rep(0.0, nStt)
    v1[(stateMat[1, ] != stateMat[2, ]) & (stateMat[3, ] == 1)] = 1.0     
    jmpPrbsPhase = v1 %*% expCounts
                   
    v1 = rep(0.0, nStt)
    v1[(stateMat[6, ] > 1)] = 1.0
    v1[(stateMat[7, ] > 1)] = v1[(stateMat[7, ] > 1)] + 1.0     
    jmpPrbsNoclipSw = v1 %*% expCounts
    
    if (mdl$controlObj$revSnpOrd) {
      jmpPrbsPhase = rev(jmpPrbsPhase) 
      jmpPrbsGrp1 = rev(jmpPrbsGrp1) 
      jmpPrbsGrp2 = rev(jmpPrbsGrp2)
      jmpPrbsNoclipSw = rev(jmpPrbsNoclipSw)
    }
  } else {
    jmpPrbsGrp1 = jmpPrbsGrp2 = jmpPrbsPhase = NULL
  }
  if (mdl$controlObj$revSnpOrd) {
    i = rev(1:mdl$nSnp)
    ancPrbs = ancPrbs[, i]
  }

  return(list(margLik = margLik, ancPrbs = ancPrbs,
              jmpPrbsGrp1 = jmpPrbsGrp1,
              jmpPrbsGrp2 = jmpPrbsGrp2,
              jmpPrbsPhase = jmpPrbsPhase))
}

AradmxJuncTree = function(mdl, phased, viterbi, reducedStateMat) {
  if (!phased) {
    if (reducedStateMat && !viterbi) {
      return(mdl$unphJtRed)
    } else {
      return(mdl$unphJt)
    }
  } else {
    return(mdl$phJt)
  }
}

AradmxMargAnc = function(mdl, inpHap1, inpHap2 = NULL, jumpProb = NULL, admixTime = NULL, 
                             snpDists = NULL, grpPriors = NULL,
                             verbose = FALSE, filtSnps = NULL,
                             moreExpectedCounts = FALSE,
                             reducedStateMat = TRUE,
                             viterbi = FALSE) {
  if (mdl$isBiDirectional) {
    r = list()
    for (nm in c("fwdMdl", "revMdl")) {
      r[[nm]] = AradmxMargAnc(mdl[[nm]], inpHap1, inpHap2 = inpHap2, jumpProb = jumpProb,
                           admixTime = admixTime, snpDists = snpDists,
                           grpPriors = grpPriors, verbose = verbose,
                           filtSnps = filtSnps, moreExpectedCounts = moreExpectedCounts,
                           reducedStateMat = reducedStateMat,
                           viterbi = viterbi)
    }
    if ("ancPrbs" %in% names(r$fwdMdl) && !is.null(r$fwdMdl$ancPrbs)) {
      r$ancPrbs = (r$revMdl$ancPrbs + r$fwdMdl$ancPrbs) / 2
    } 
    return(r)
  }
  if (!is.integer(inpHap1)) {
    inpHap1 = as.integer(inpHap1)
  }
  if (!is.null(inpHap2) && !is.integer(inpHap2)) {
    inpHap2 = as.integer(inpHap2)
  }
  
  if ("mdls" %in% names(mdl)) {
    mdls = mdl$mdls
    nMdl = length(mdls)
    for (k in 1:nMdl) {
      a1 = AradmxMargAnc(mdls[[k]], inpHap1, inpHap2 = inpHap2,
                             jumpProb = jumpProb, admixTime = admixTime, 
                           snpDists = snpDists, grpPriors = grpPriors,
                           verbose = verbose, filtSnps = filtSnps,
                           reducedStateMat = reducedStateMat)
      if (k == 1) r = a1
      if (k > 1) {
        r$ancPrbs = r$ancPrbs + a1$ancPrbs
        r$margLik = c(r$margLik, a1$margLik)
      }
    }
    r$ancPrbs = r$ancPrbs / nMdl
    return(r)
  }
  
  if (mdl$controlObj$revSnpOrd) {
    inpHap1 = ReverseSnpOrd(inpHap1)
    if (!is.null(inpHap2)) inpHap2 = ReverseSnpOrd(inpHap2)
  }
  if (viterbi && reducedStateMat) {
    warning(paste("AradmxUnphasedAnc : reducedStateMat overriden and set",
                  "to FALSE for viterbi path"))
    reducedStateMat = FALSE
  }
  jt = AradmxJuncTree(mdl, is.null(inpHap2), viterbi, reducedStateMat)
  
  dropLookups = mdl$dropLookups
  maxDrop = mdl$maxDrop # dim(dropLookups)[2]
  binTrees = mdl$binTrees
  
  nSnp = length(inpHap1)
  if (nSnp != mdl$nSnp) {
    stop("input haplotypes/genotypes have incorrect number of snps")
  }
  
  interGrpTrans = GetJumpProbs(nSnp, jumpProb = jumpProb, admixTime = admixTime,
                               snpDists = snpDists,
                               revSnpOrd = mdl$controlObj$revSnpOrd)
  if(is.null(grpPriors)){
    grpPriors = rep(1/mdl$nPop, mdl$nPop)
  }
  
  stateMat = jt$stateMat
  nStt = ncol(stateMat)
  
  # TODO : rewrite AradmxUnphasedFwdBkwd so that it does not take 'skipPsetPrbs'
  # and so that it takes the model object rather than the trees and drop-lookups
  # separately
  if (is.null(filtSnps)) {
    skipPsetPrbs = rep(as.integer(0), nSnp)
  } else {
    skipPsetPrbs = (!filtSnps)
  }

  if (viterbi) {   
    vitTrace = rep(as.integer(0), length(inpHap1))       
    margLik = ancPrbs = expCounts = NULL
  } else {
    margLik = 0.0                    

    if (!is.null(inpHap2)) {                      
      ancPrbs = rep(0, mdl$nPop * nSnp)
      dim(ancPrbs) = c(mdl$nPop, nSnp)
    } else {
      ancPrbs = rep(0, mdl$nPop * nSnp)
      dim(ancPrbs) = c(mdl$nPop, nSnp)
    }
    if (moreExpectedCounts) {
      expCounts = matrix(nrow = nStt, ncol = nSnp, data = 0.0)
    } else {
      expCounts = NULL
    }
    vitTrace = NULL
  }
  if (!is.null(inpHap2)) {
    checkPointed = FALSE
    popLabels = rep(TRUE, mdl$nPop)
    .Call("AradmxUnphApproxFwdBkwd_RI", inpHap1, inpHap2,
          dropLookups, maxDrop, jt, maxDrop, interGrpTrans, grpPriors, 
          mdl$binTreesPacked, popLabels, checkPointed, margLik, ancPrbs,
          PACKAGE = kPhaseEmPkgName) 
    
    if (viterbi) {               
      return(AradmxUnphasedVitRet(mdl, vitTrace, statMat))
    } else {
      return(AradmxUnphasedMargRet(mdl, margLik, ancPrbs, expCounts, stateMat,
                                   moreExpectedCounts))
    }
  } else {         
    checkPointed = FALSE
    popLabels = rep(TRUE, mdl$nPop)
    .Call("AradmxPhasedExactFwdBkwd_RI", inpHap1,
          dropLookups, maxDrop, jt, maxDrop, interGrpTrans, grpPriors, 
          mdl$binTreesPacked, popLabels, checkPointed, margLik, ancPrbs,
          PACKAGE = kPhaseEmPkgName) 
  		 
    if (viterbi) {
      return(AradmxPhasedVitRet(mdl, vitTrace, jt$stateMat))
    } else {
      return(AradmxPhasedMargRet(mdl, margLik, ancPrbs, expCounts, jt$stateMat,
                                 moreExpectedCounts))
    }
  }
}


################################################################################
## Experimental things                                                     
################################################################################



AradmxModelIncCounts = function(haps, masks, popIdx, modelObj, incAmt = 1.0) {
  stop("function deprecated and currently not working")
  if (modelObj$controlObj$revSnpOrd) {
    i1 = rev(1:nrow(haps))
    haps = haps[i1, ]          
    masks = masks[i1, ]
  }
  
  binTrees = modelObj$binTrees    
  maxDrop = length(binTrees[[1]])
  nPop = modelObj$nPop
  nSnp = length(binTrees[[1]][[1]]$splitLists)
  
  if (nrow(haps) != nSnp || nrow(masks) != nSnp) {
    stop("haps/masks must have dimensions [# snp's] X [# chromosomes]")
  } else if (ncol(haps) != ncol(masks)) {
    stop("haps/masks must have dimensions [# snp's] X [# chromosomes]")
  }
  
  z = binTrees[[popIdx]]
  for (dropIdx in 1:length(z)) {
    z1 = z[[dropIdx]][[3]]
    for (snpIdx in 1:length(z1)) {
      z1[[snpIdx]][] = z1[[snpIdx]][] + 1e-9
    }
    z[[dropIdx]][[3]] = z1
  }
  binTrees[[popIdx]] = z

  .Call("AradmxIncCountsMaskedRI", haps, masks, popIdx-1, maxDrop, binTrees,
        incAmt, PACKAGE = kPhaseEmPkgName) 
            
  modelObj$binTrees = binTrees
  modelObj = AradmxMakePackedTrees(modelObj)
  return(modelObj)
}

AradmxCheckBinTrees = function(psets) {
  for (k in 1:length(psets)) {
    p0 = psets[[k]]
    p0L = p0[[2]]
    p0C = p0[[3]]

    for (j in 1:length(p0L)) {
      print("------------------------------")
      print(c(k,j))
      print(p0L[[j]])
      print(p0C[[j]])
      print("-  -  -  -  -  -  -  -  -  -  -")
      AradmxTraverseBinTree(p0L[[j]], p0C[[j]], 1, j)
    }
  }
}

AradmxProbsCounts = function(mdl, hap) {
  nSnp = length(hap)
  
  if (mdl$controlObj$revSnpOrd) {
    hap = rev(hap)
  }
  
  prbs = matrix(nrow = mdl$nPop, ncol = nSnp, data = 0)
  cts = matrix(nrow = mdl$nPop, ncol = nSnp, data = 0)
  for (popIdx in 1:mdl$nPop) {    
    c1 = rep(0.0, nSnp)
    p1 = rep(0.0, nSnp)
    .Call("AradmxCondProbsCounts", hap, popIdx-1, mdl$binTrees, p1, c1,
          PACKAGE = kPhaseEmPkgName)
    prbs[popIdx,] = p1       
    cts[popIdx,] = c1
  }
  
  
  if (mdl$controlObj$revSnpOrd) {
    i1 = rev(1:nSnp)
    prbs = prbs[, i1]
    cts = cts[, i1]
  }
  
  return(list(prbs = prbs, cts = cts))
}

AradmxCollapseTrees = function(ancDbs, mdl, scoreType = kBicScore) {
  mdl2 = AradmxDeepCopyTrees(mdl)
  nPop = length(mdl$binTrees)
  
  for (popIdx in 1:mdl$nPop) {
    if (scoreType == kBicScore) {
      splitPenalty = -0.5 * log(nrow(ancDbs[[popIdx]]))
    } else if (scoreType == kAicScore) {
      splitPenalty = -2
    } else {
      splitPenalty = 0
    }
    .Call("BinTreeCollapse", mdl2$binTrees, splitPenalty,
          mdl$controlObj$pCount, popIdx-1,
          PACKAGE = kPhaseEmPkgName)
  }
  mdl$binTreesUncollapsed = mdl$binTrees
  mdl$binTrees = mdl2$binTrees
  mdl = AradmxMakePackedTrees(mdl)
  return(mdl)
}

AradmxDeepCopyTrees = function(mdl) {
  # many of the C functions modify the data and so we often want to make a copy
  # of the trees and counts before running them
  binTree2 = mdl$binTrees
  nPop = length(binTree2)
  for (popIdx in 1:nPop) {
    for (dropIdx in 1:length(binTree2[[popIdx]])) {
      splitCounts = binTree2[[popIdx]][[dropIdx]][[3]]
      splitLists = binTree2[[popIdx]][[dropIdx]][[2]]
      
      for (k in 1:length(splitLists)) {
        splitCounts[[k]] = splitCounts[[k]] + 1e-300
        splitCounts[[k]] = splitCounts[[k]] - 1e-300
        splitLists[[k]] = splitLists[[k]] + as.integer(0)
      } 
                                                    
      binTree2[[popIdx]][[dropIdx]][[2]] = splitLists
      binTree2[[popIdx]][[dropIdx]][[3]] = splitCounts
    } 
  }
  mdl$binTrees = binTree2
  return(mdl)
}

AradmxTiedParams = function(mdl, weighted = TRUE, bfPcount = NULL, bayesFactThresh = 0.0) {
  mdl = AradmxDeepCopyTrees(mdl)
  binTree2 = mdl$binTrees
  nPop = length(binTree2)
  
  nSnp = length(binTree2[[1]][[1]][[3]])
  
  if (is.null(bfPcount)) {
    bfPcount = mdl$pCount
  }
  if (bfPcount <= 0) {
    stop("bfPcount must be strictly positive")
  }
  
  maxBayesFacts = rep(0, nSnp)
  nTied = rep(as.integer(-1), nSnp)
  nUntied = rep(as.integer(-1), nSnp)
  
  .Call("BinTreeBayesFactSnap", binTree2, mdl$pCount, bfPcount,
        bayesFactThresh, maxBayesFacts, nTied, nUntied, weighted,
        PACKAGE = kPhaseEmPkgName)
        
  tieRes = list(bfPcount = bfPcount, bayesFactThresh = bayesFactThresh,
                nTied = nTied, nUntied = nUntied,
                maxBayesFacts = maxBayesFacts)
  mdl$tieRes = tieRes
  mdl$binTrees = binTree2
  return(mdl)
}


AradmxPhasedMargLiks = function(mdl, admixDb, admixTimes, ...) {
  idx = seq(from = 1, to = nrow(admixDb), by = 1)
  
  r = matrix(nrow = length(idx), ncol = length(admixTimes), data = 0)
  for (k in 1:length(admixTimes)) {
    print(k)
    for (j in idx) {
      h1 = admixDb[j,]
      r[j, k] = AradmxPhasedAnc(mdl, admixDb[j,], admixTimes[k])$margLik
    }
  }
  return(r)
}


AradmxUnphasedMargLiks = function(mdl, admixDb, admixTimes, ...) {
  idx = seq(from = 2, to = nrow(admixDb), by = 2)
  
  r = matrix(nrow = length(idx), ncol = length(admixTimes), data = 0)
  for (k in 1:length(admixTimes)) {
  for (j in idx) {
    h1 = admixDb[j-1,]
    h2 = admixDb[j,]
    
    r[j/2, k] = AradmxUnphasedAnc(mdl, h1, h2, admixTimes[k])$margLik
  }}
  return(r)
}

AradmxGetTree = function(mdl, popIdx, snpIdx, dropIdx) {

  TreeTrav = function(splitStream, counts, idx) {
    if (splitStream[idx] == -1) {
      c0 = counts[splitStream[idx+1]+1]
      c1 = counts[splitStream[idx+2]+1]
      p1 = c1 / (c1 + c0)
      return(c(p1, c0, c1))
    } else {
      v1 = TreeTrav(splitStream, counts, 1 + splitStream[idx+1])
      v2 = TreeTrav(splitStream, counts, 1 + splitStream[idx+2])

      return(list(v1, v2))
    }
  }

  splitStream = mdl$binTrees[[popIdx]][[dropIdx]][[2]][[snpIdx]]
  splitCounts = mdl$binTrees[[popIdx]][[dropIdx]][[3]][[snpIdx]]

  tr = TreeTrav(splitStream, splitCounts, 1)
  return(list(splitStream = splitStream, splitCounts = splitCounts,
              tree = tr))
}

AradmxTreeSummary = function(mdl, snpIdx, dropIdx = 1) {
  nPop = length(mdl$binTrees)

  for (popIdx in 1:nPop) {
    tr = AradmxGetTree(mdl, popIdx, snpIdx, dropIdx)
    print(tr$tree)
  }
}



AradmxTraverseBinTree = function(splitList, leafCounts, startIdx, maxVar) {
  if (startIdx <= 0 || startIdx >= length(splitList)) {
    stop("invalid split index")
  }

  if (splitList[startIdx] < 0) {
    # it's a leaf
    ct1 = splitList[startIdx+1] + 1
    ct2 = splitList[startIdx+2] + 1
    if (ct1 <= 0 || ct1 > length(leafCounts)) stop("invalid count index")
    if (ct2 <= 0 || ct2 > length(leafCounts)) stop("invalid count index")
    print(paste(startIdx, "leaf"))
    print(c(ct1, ct2))
    print(leafCounts[c(ct1, ct2)])

  } else {
    if (splitList[startIdx] > maxVar - 1e-6) {
      stop("invalid variable")
    }
    print(paste(startIdx, "non-leaf"))

    nd1 = splitList[startIdx+1]
    nd2 = splitList[startIdx+1]
    AradmxTraverseBinTree(splitList, leafCounts, nd1+1, maxVar)
    AradmxTraverseBinTree(splitList, leafCounts, nd2+1, maxVar)
  }
}