FphOutput = function(x, fname, scramble = TRUE) {
  # This function writes x to file in a format that fastPHASE
  # can read.  If 'scramble' is TRUE, we randomize the phasing
  # before writing.  The file format uses question marks (?)
  # for missing alleles (x[i,j] == 2)
  if (scramble) {
    x = RandPhasing(x)
  }
  if ((nrow(x) %% 2) != 0) {
    stop("x must have even number of rows")
  }
  
  M = nrow(x) / 2
  N = ncol(x)
	headerStr = paste(nrow(x)/2, N, sep = "\n")
	write(headerStr, file = fname)
	i1 = seq(from = 2, to = (2*M), by = 2) - 1

	for (k in i1) {
		v1 = x[k,]
		v2 = x[k+1,]
		write(paste("# id", (k+1)/2), append=TRUE, file=fname)

		if (sum(v1 > 1) > 0) {
			v1[v1 > 1] = "?"
		}
		if (sum(v2 > 1) > 0) {
			v2[v2 > 1] = "?"
		}
		write(v1, sep="", append=TRUE, file=fname, ncolumns=N)
		write(v2, sep="", append=TRUE, file=fname, ncolumns=N)
	}
}

FphWrapper = function(x, nClust = 20, nEM = 35,
                      nRestart = 15, pathBase = NULL,
                      phasedRefPanel = NULL,
                      cmdPrefix = "",
                      removeIntermediate = TRUE) {
  x = MatrixSlice(x, matType = "IntMat")
  if (is.null(pathBase)) {
    randPathBase = TRUE
    pathBase = sprintf("fph_tmp_%d_", as.integer(runif(1) * 100000))
  } else {
    randPathBase = FALSE
  }
  
  while (TRUE) {
    inpFname = paste(pathBase, ".inp", sep = "")
    outFname = paste(pathBase, ".out", sep = "")
    
    if (randPathBase && (FileExists(inpFname) || FileExists(outFname))) {
      pathBase = sprintf("fph_tmp_%d_", as.integer(runif(1) * 100000))
      print(sprintf("old paths were taken.  generating new ones: %s",
                    pathBase))
    } else {
      break
    }
  }
  FphOutput(x, inpFname)

  fphArgs = sprintf("-K%d -T%d -C%d -o%s", nClust, nRestart, nEM, outFname)
                    
  phasedRefPanelFnm = NULL
  markersFnm = NULL
  
  if (!is.null(phasedRefPanel)) {
    phasedRefPanelFnm = paste(pathBase, ".refPanel", sep = "")
    
    FphOutput(phasedRefPanel, phasedRefPanelFnm, scramble = FALSE)                                                        
    fphArgs = sprintf("%s -b%s", fphArgs, phasedRefPanelFnm)
  }
  
  fphArgs = sprintf("%s %s", fphArgs, inpFname)
  fphCmd = sprintf("%s%s %s %s",
                   cmdPrefix,
                   ifelse(IsWindows(), "./fastPHASE.exe", "./fastPHASE"),
                   fphArgs, inpFname)

  system(fphCmd)

  haps = FphInput(paste(outFname, "_hapguess_switch.out", sep = ""), nrow(x)/2)
  if (removeIntermediate) {
    system(paste("rm ", outFname, "_hapguess_switch.out", sep = ""))
    system(paste("rm ", outFname, "_finallikelihoods", sep = ""))
    system(paste("rm ", inpFname))
    
    if (!is.null(phasedRefPanel)) {
      system(paste("rm", phasedRefPanelFnm))
    }
  }
  return(haps)                      
}

FphInput = function(fname, nSamp) {
	ffLines = read.table(file = fname, header = FALSE, as.is = TRUE,
                       sep = "$", comment.char = "")
	
	retMat = NULL
	j = 1
	while (j <= nrow(ffLines)) {
    firstChar = substr(ffLines[j,1], 1, 1)
    if (firstChar == "#") {
      j = j + 1
      h1 = as.integer(strsplit(ffLines[j,1], " ")[[1]])
      j = j + 1
      h2 = as.integer(strsplit(ffLines[j,1], " ")[[1]])
      
      if (is.null(retMat)) {
        retMat = rbind(h1, h2)
      } else {
        retMat = rbind(retMat, h1, h2)
      }
    } else {
      j = j + 1
    }
  }
	return(retMat)
}