HrcWrapper = function(db, modelType = c("VMM", "VMM_AVG", "S"),
                      w = 100, o = 10, r = 10,
                      pathBase = NULL, 
                      removeIntermediate = TRUE,
                      callPrefix = "", dontRun = FALSE, readOnly = FALSE,
                      tmpDir = NULL) {
  # see haplorec documentation for meaning of 'w', 'o' and 'r'
  # these are window the window sizes used to split up the dataset
  
  if (is.null(pathBase)) {
    pathBase = sprintf("hrc_tmp_%d_", as.integer(runif(1) * 100000))
  }
  
  db = MatrixSlice(db, matType = "IntMat")
  nHap = nrow(db)
  
  x = db + as.integer(1)
  x[db == 3] = 0
  x = matrix(nrow = nrow(x), data = as.character(x))
  x = rbind(paste("M", 1:ncol(x), sep = ""), x)
  x = cbind(c("Id", rep(1:(nHap/2), each = 2)), x)
  
  inpFname = paste(pathBase, ".unphInp", sep = "")
  write.table(x, sep = " ", row.names = FALSE, col.names = FALSE,
              file = inpFname, quote = FALSE)

  javaArgs = "-Xmx1000m -jar HaploRec/HaploRec.jar"
  hrcArgs = ""
  
  if (!is.null(modelType) && length(modelType) > 0) {
    hrcArgs = sprintf("%s -p %s", hrcArgs, modelType[1])
  }
  
  hrcArgs = sprintf("%s %s", hrcArgs, inpFname)
  
  if (!is.null(w) && w < ncol(db)) {
    hrcArgs = sprintf("%s %d %d %d", hrcArgs, w, o, r)
  }
                  
  if (!is.null(tmpDir)) {
    javaArgs = paste("-Djava.io.tmpdir=", tmpDir, " ", javaArgs, sep = "")
  }

  hrcCmd = paste(callPrefix, "java ", javaArgs, " ", hrcArgs, sep="")
  if (dontRun) {
    return(hrcCmd)
  } else if (!readOnly) {
    system(hrcCmd)
  }

  tmpFile = paste(inpFname, ".id_and_status.tmp", sep = "")
  resultFile = paste(inpFname, ".reconstructed", sep = "")
  
  x1 = read.table(file = resultFile, sep = " ", header = FALSE, as.is = TRUE)
  x1 = as.matrix(x1)
  
  x1 = x1[-1, -1]
  x1 = matrix(nrow = nrow(x1), data = as.integer(x1))
  x1 = x1 - as.integer(1)
  x1[x1 < 0] = as.integer(2)
  
  if (removeIntermediate) {    
    system(sprintf("rm %s*", pathBase))
  }
  return(x1)
}