#ifndef ARADMX_HPP_
#define ARADMX_HPP_

#include "util.hpp"

namespace Aradmx {

class PackedTreeIndexer {
public:
	PackedTreeIndexer() {
	  n_pops_ = n_snps_ = n_mdls_ = -1;
	  idx_ = snp_idx_ = pop_idx_ = mdl_idx_ = max_idx_ = 0;
	}
	PackedTreeIndexer(int n_pops, int n_snps, int n_mdls) {
	  this->Init(n_pops, n_snps, n_mdls);
	}

	void Init(int n_pops, int n_snps, int n_mdls) {
	  n_pops_ = n_pops;
	  n_snps_ = n_snps;
	  n_mdls_ = n_mdls;
	  max_idx_ = n_pops_ * n_snps_ * n_mdls_;
	  idx_ = snp_idx_ = pop_idx_ = mdl_idx_ = 0;
	}
	int PackedPositionIndex(int pop_idx, int mdl_idx, int snp_idx) const {
    return(pop_idx + (mdl_idx + snp_idx * n_mdls_) * n_pops_);
	}
	void StartIterator() {
		idx_ = snp_idx_ = pop_idx_ = mdl_idx_ = 0;
	}
	void IncIterator() {
		++pop_idx_;
		++idx_;
		if (pop_idx_ >= n_pops_) {
			pop_idx_ = 0;
			++mdl_idx_;
			if (mdl_idx_ >= n_mdls_) {
				mdl_idx_ = 0;
				++snp_idx_;
			}
		}
	}
	bool Finished() const {
		return (idx_ >= max_idx_);
	}
	int CurrentPackedPositionIndex() const {
    return(pop_idx_ + (mdl_idx_ + snp_idx_ * n_mdls_) * n_pops_);
	}
	int snp_idx() const { return snp_idx_; }
	int mdl_idx() const { return mdl_idx_; }
	int pop_idx() const { return pop_idx_; }

protected:
	int n_pops_, n_mdls_, n_snps_;
	int idx_, snp_idx_, pop_idx_, mdl_idx_, max_idx_;
};

class DropLookupIndexer {
public:
	DropLookupIndexer() {
		this->Init(0, 0, 0);
	}
	DropLookupIndexer(int n_pops, int n_snps, int max_cut) {
		this->Init(n_pops, n_snps, max_cut);
	}

	void Init(int n_pops, int n_snps, int max_cut) {
		max_cut_ = max_cut;
		n_pops_ = n_pops;
		n_snps_ = n_snps;
	}

	int FlatIndex(int pop_idx, int snp_idx, int cut_idx) const {
		return(pop_idx + (cut_idx + snp_idx * max_cut_) * n_pops_);
	}
protected:
	int n_pops_, max_cut_, n_snps_;
};

}
#endif /*ARADMX_HPP_*/
