#ifndef ARADMX_PACK_TREES_HPP_
#define ARADMX_PACK_TREES_HPP_

#include "util.hpp"
#include "aradmx.hpp"

namespace Aradmx {
namespace PackTrees {

// If we add canaries to the trees, we put the following into the stream
// before the splits :
//  position 0 : -123123   (indicates a canary is present)
//  position 1 : group-index
//  position 2 : snp-index
//  position 3 : drop-index

const int kCanaryCode = -123123;
const int kCanarySize = 4;

__inline bool HasCanary(int * split_stream) {
  return((split_stream[0] == kCanaryCode) ? true : false);
}

__inline int GetCanarySnpIndex(int * split_stream) {
  return(split_stream[2]);
}

__inline bool CheckCanarySnpIndex(int snp_idx, int * split_stream) {
  return((split_stream[2] == snp_idx) ? true : false);
}

__inline int * ShiftIfCanary(int * split_stream) {
  if (HasCanary(split_stream)) {
  	return(split_stream + 4);
  } else {
  	return(split_stream);
  }
}

void PackCounts(int offset, int * split_stream,
		            const Util::RealVec * count_table,
		            bool count_pack, int * packed_tree, int * open_posn);

int NumNodes(int offset, int * split_stream);

void CountsToProbs(int offset, double pcount, int * split_stream);

int * ShiftIfCanary(int * split_stream);

SEXP GetModelsForAnc(SEXP mdl_trees, int anc_idx);

SEXP GetModelElt(SEXP mdl_trees, int anc_idx, int mdl_idx, const char* nm);

SEXP GetModelEltForSnp(SEXP mdl_trees, int anc_idx, int mdl_idx,
		                     int snp_idx, const char* nm);

SEXP GetSplitLists(SEXP mdl_trees, int anc_idx, int mdl_idx);

SEXP GetSplitListsForSnp(SEXP mdl_trees, int anc_idx, int mdl_idx,
		                     int snp_idx);

SEXP GetLeafCountsForSnp(SEXP mdl_trees, int anc_idx, int mdl_idx,
		                     int snp_idx);

SEXP GetLeafCounts(SEXP mdl_trees, int anc_idx, int mdl_idx);

void NumPopsModelsSnps(SEXP mdl_trees, int* n_pop, int* n_snp, int* n_mdl,
		                   bool check_model);

class PackedModelIndexer : public PackedTreeIndexer {
public:
	PackedModelIndexer(SEXP bin_trees_sxp) : bin_trees_sxp_(bin_trees_sxp) {
		int n_pops = 0, n_snps = 0, n_mdls = 0;
	  NumPopsModelsSnps(bin_trees_sxp_, &n_pops, &n_snps, &n_mdls, false);
	  this->Init(n_pops, n_snps, n_mdls);
	}
	SEXP CurrentSplitListsForSnp() {
		return(GetSplitListsForSnp(bin_trees_sxp_, pop_idx_, mdl_idx_, snp_idx_));
	}
	SEXP CurrentLeafCountsForSnp() {
		return(GetLeafCountsForSnp(bin_trees_sxp_, pop_idx_, mdl_idx_, snp_idx_));
	}
protected:
	SEXP bin_trees_sxp_;
};

}  // namespace PackTrees
}  // namespace Aradmx

extern "C" {

SEXP AradmxPackInitTrees_RI(SEXP bin_trees_sxp, SEXP tree_posns_sxp,
		                        SEXP count_pack_sxp, SEXP zero_counts_sxp,
		                        SEXP add_canaries_sxp);

SEXP AradmxCtTreesToPrbTrees_RI(SEXP bin_trees_sxp, SEXP packed_trees_sxp,
		                            SEXP tree_posns_sxp,
		                            SEXP pcount_sxp);

SEXP AradmxPackDropLookups_RI(SEXP dropLookupsSxp);

}  // extern "C"

#endif /*ARADMX_PACK_TREES_HPP_*/
