#ifndef ARADMX_UNPHASED_APPROX_HPP_
#define ARADMX_UNPHASED_APPROX_HPP_


#include "util.hpp"
#include "aradmx_fwd_bkwd.hpp"
#include "aradmx.hpp"
#include "aradmx_unph_statemat.hpp"


//#define EXACT_INF_DEBUG_CHECKS 1
namespace Aradmx {
namespace UnphApprox {

using namespace Util;
using namespace Aradmx::FwdBkwd;

class State : public UnphStatemat::UnphApproxState {
public:
	UnphStatemat::Field jump_idx_, anc_jump_idx_[2];

public:
  virtual void FromBuf(int * buf, int win_sz) {
    this->UnphStatemat::UnphApproxState::FromBuf(buf);

    if (jmp_idx_ == 0) jump_idx_ = win_sz - 1;
    if (jmp_idx_> 0) jump_idx_ = jmp_idx_ - 1;
    anc_jump_idx_[0] = (strands_[0].anc_jmp_) ? jump_idx_ : (win_sz - 1);
    anc_jump_idx_[1] = (strands_[1].anc_jmp_) ? jump_idx_ : (win_sz - 1);
  }

	static vector<State> StatesFromBuf(int * buf, int n_states, int win_sz) {
		vector<State> r(n_states);
	  for (int i = 0; i < n_states; ++i, buf += kStateSize) {
	    r[i].FromBuf(buf, win_sz);
	  }
	  return r;
	}
};

class UnphasedApproxModelParams : public ModelParams {
public:
	typedef vector<State> StateMat;

	UnphasedApproxModelParams() : ModelParams() {
	  state_size_ = 0;
	  haps_[0] = haps_[1] = NULL;
		hap_pair_ = HapPair(NULL, NULL);
	}

	void set_haps(Hap& h1, Hap& h2) {
		haps_[0] = h1;
		haps_[1] = h2;

		hap_pair_ = HapPair(h1, h2);
	}

	void Fill(SEXP drop_lookups_sxp, SEXP max_drop_sxp,
	          SEXP state_mat_sxp, SEXP win_sz_sxp,              // The state matrix
	          SEXP L2RMap, SEXP R2LMap,     // mappings to absorb messages
	          SEXP anc_transprb_sxp, SEXP anc_priors_sxp, // Recombination and ancestral prior parameters
	          SEXP bin_trees_sxp);

	void FirstSnpZero(double * r);
	void GetAncPrbs(int snp_idx, const double * cur_pttl, double * anc_prbs) const;
	void CondProbs(const IntVec& pop_lab, int snp_idx, double * r);
	void ObservePhasing(int snp_idx, int phase_or, double * r) const {}

	void ClearTreeLookups();
public:
  Hap haps_[2];
  HapPair hap_pair_;
  int state_size_;
  StateMat state_mat_;

  vector<const int*> tree_lookup_;
};

}  // namespace UnphApprox {
}  // namespace Aradmx {


extern "C" {

	SEXP AradmxUnphApproxFwdBkwd_RI(SEXP hap1_sxp, SEXP hap2_sxp, SEXP drop_lookups_sxp, SEXP max_drop_sxp,
	                              SEXP junc_tree_sxp, SEXP win_sz_sxp,              // The state matrix
	                              SEXP anc_transprb_sxp, SEXP anc_priors_sxp, // Recombination and ancestral prior parameters
	                              SEXP bin_trees_sxp,    // Parent sets and their observed counts/multinomial parameters
	                              SEXP pop_lab_sxp,
	                              SEXP checkpointed_sxp,
	                              SEXP marg_lik_sxp, SEXP anc_prbs_sxp);

}  // extern "C"


#endif /*ARADMX_UNPHASED_APPROX_HPP_*/
