#include "binary_mat.hpp"

namespace BinaryMatrix {

bool IsCustomMat(SEXP sxp_obj) {
  if (-1 == ListEltIdxByName(sxp_obj, "nr")) return(false);
  if (-1 == ListEltIdxByName(sxp_obj, "nc")) return(false);
  return(true);
}

bool IsPackedMat(SEXP sxp_obj) {
  if (!IsCustomMat(sxp_obj)) return(false);
  if (-1 == ListEltIdxByName(sxp_obj, "dat")) return(false);
  if (-1 == ListEltIdxByName(sxp_obj, "nri")) return(false);
  return(true);
}

bool CheckName(SEXP sxp_obj, const char * type_name) {
  SEXP type_sxp = ListEltByName(sxp_obj, "type", NULL);

  const char * type_str = CHAR(STRING_ELT(type_sxp, 0));
  if (strcmp(type_str, type_name) != 0) {
  	return(false);
  } else {
    return(true);
  }
}

bool CheckType(SEXP sxp_obj, const char * type_name) {
  if (!IsPackedMat(sxp_obj)) {
  	return(false);
  }

  return(CheckName(sxp_obj, type_name));
}

OnesMat::OnesMat(SEXP sxp) {
  SEXP nr = ListEltByName(sxp, "nr", NULL);
  SEXP nc = ListEltByName(sxp, "nc", NULL);
  nr_ = GetInt(nr, 0, NULL);
  nc_ = GetInt(nc, 0, NULL);
}

bool OnesMat::IsOnesMat(SEXP sxp_obj) {
	return(IsCustomMat(sxp_obj) && CheckName(sxp_obj, "OnesMat"));
}

ByteMat::ByteMat(SEXP sxp) {
  SEXP dat = ListEltByName(sxp, "dat", NULL);
  SEXP nr = ListEltByName(sxp, "nr", NULL);
  SEXP nc = ListEltByName(sxp, "nc", NULL);
  SEXP nri = ListEltByName(sxp, "nri", NULL);

  dat_ = (unsigned char *)INTEGER(dat);
  nr_ = GetInt(nr, 0, NULL);
  nc_ = GetInt(nc, 0, NULL);
  bytes_per_col_ = 4 * GetInt(nri, 0, NULL);
}

bool ByteMat::IsByteMat(SEXP sxp_obj) {
	return(CheckType(sxp_obj, "ByteMat"));
}

BinaryMat::BinaryMat(SEXP sxp_obj) {
  SEXP dat = ListEltByName(sxp_obj, "dat", NULL);
  SEXP nr = ListEltByName(sxp_obj, "nr", NULL);
  SEXP nc = ListEltByName(sxp_obj, "nc", NULL);
  SEXP nri = ListEltByName(sxp_obj, "nri", NULL);

  dat_ = (unsigned char *)INTEGER(dat);
  nr_ = GetInt(nr, 0, NULL);
  nc_ = GetInt(nc, 0, NULL);
  bytes_per_col_ = 4 * GetInt(nri, 0, NULL);
}

bool BinaryMat::IsBinaryMat(SEXP sxp_obj) {
	return(CheckType(sxp_obj, "BinaryMat"));
}

bool IsBinaryMatScaled(SEXP sxp_obj) {
	return(CheckType(sxp_obj, "BinaryMatScaled"));
}

bool IsByteMatScaled(SEXP sxp_obj) {
	return(CheckType(sxp_obj, "ByteMatScaled"));
}

void BinaryMat::SetElt(int ri, int ci, int val) {
  int ri0 = ri / 8;

  unsigned char * d = dat_ + (ri0 + bytes_per_col_ * ci);
  int ri1 = ri % 8;
  unsigned char msk = ~(1 << ri1);
  *d &= msk;
  *d |= (val << ri1);
}

int BinaryMat::GetElt(int ri, int ci) const {
#ifdef BM_BOUNDCHECKS
	if (ri < 0 || ri >= this->nr_ || ci < 0 || ci >= this->nc_) {
		error("ScaledMat::GetEltDbl : bad row/column indices\n");
	}
#endif
  int ri0 = ri / 8;

  int d = dat_[ri0 + bytes_per_col_ * ci];
  int ri1 = ri % 8;
  int msk = (1 << ri1);
  return((d & msk) ? 1 : 0);
}

void BinaryMat::RowIterator::SetBuf(unsigned char x) {
  buf_[0] = (x & 1) ? 1 : 0;
  buf_[1] = (x & 2) ? 1 : 0;
  buf_[2] = (x & 4) ? 1 : 0;
  buf_[3] = (x & 8) ? 1 : 0;
  buf_[4] = (x & 16) ? 1 : 0;
  buf_[5] = (x & 32) ? 1 : 0;
  buf_[6] = (x & 64) ? 1 : 0;
  buf_[7] = (x & 128) ? 1 : 0;
}

void BinaryMat::RowIterator::Inc() {
  //++idx_;
  ++buf_idx_;
  if (buf_idx_ == 8) {
    ++x_idx_;
    buf_idx_ = 0;
    this->SetBuf(x_[x_idx_]);
  }
}

}  // namespace
