#include "aradmx_unph_statemat.hpp"
#include "util.hpp"

#define EXACT_INF_DEBUG_CHECKS 1
namespace Aradmx {
namespace UnphStatemat {
//#define ABBN_REDUCE_STATEMAT 1
// jmpIdx = 0 indicates that there is no transition of any kind
// jmpIdx = k, indicates that there was a transition between indices
//    (v - k) and (v + 1 - k)


void UnphApproxFillSxp(bool reduced, int n_groups, int win_sz,
                            int * n_elt, vector<UnphApproxState> * states,
                            SEXP * sxp_states) {
#ifdef EXACT_INF_DEBUG_CHECKS
	Util::LogToFileSprintf("C:/Users/n/Documents/Rpackages/log.txt", 0,
                   "UnphApproxFillSxp: entering\n");
	Util::LogToFileSprintf("C:/Users/n/Documents/Rpackages/log.txt", 0,
                   "reduced = %d, n_groups = %d, win_sz = %d\n",
                   (int)reduced, n_groups, win_sz);
#endif
  int n = *n_elt = UnphApproxState::GetStates(reduced, n_groups, win_sz, NULL);

#ifdef EXACT_INF_DEBUG_CHECKS
  Util::LogToFileSprintf("C:/Users/n/Documents/Rpackages/log.txt", 0,
                   "n_elt = %d\n", *n_elt);
#endif

  vector<UnphApproxState> buf(n);
  UnphApproxState::GetStates(reduced, n_groups, win_sz, &buf);

  SEXP stateMatSxp;
  PROTECT(stateMatSxp = NEW_INTEGER(UnphApproxState::kStateSize * n));
  int * stateMat = INTEGER(stateMatSxp);

  for (int i = 0; i < (*n_elt); ++i) {
  	buf[i].ToBuf(stateMat);
    stateMat += UnphApproxState::kStateSize;
  }

  Util::SetMatrixDim(stateMatSxp, UnphApproxState::kStateSize, n);
  *states = buf;
  *sxp_states = stateMatSxp;

#ifdef EXACT_INF_DEBUG_CHECKS
  Util::LogToFileSprintf("C:/Users/n/Documents/Rpackages/log.txt", 0,
                   "UnphApproxFillSxp: exiting\n");
#endif
}

//


// smap[k] contains the state with largest index that is a mirror state of
// of state 'k'.  If there is no mirror state we have smap[k] = -1
// smap[k] = j >= 0  implies that smap[j] = k, and this function counts the
// number of redudant states :  # { k : smap[k] >= k }.  This contains states
// that are their own mirror (flipping the strand leaves the state unchanged)
int NumMirrorStates(int n_states, const int * smap, int * reduced_map,
		                int * reduced_to_expanded) {
  int r = 0;
  for (int i = 0; i < n_states; ++i) {
    if (smap[i] >= i) {
      if (reduced_map) {
        reduced_map[i] = reduced_map[smap[i]] = r;
      }
      if (reduced_to_expanded) {
        reduced_to_expanded[r] = i;
      }
      r++;
    }
  }
  return r;
}

void GetReducedMap(int n_states, int * l2r_map, int l2r_map_len,
                  int * reduced_map, int n_reduced_states,
                  int * reduced_to_expanded,
                  int * l2r_map_new, int * l2r_map_new_len) {

#ifdef CONSOLE_STACK_TRACE
	Util::ConsolePrint("GetReducedMap : Entering\n");
#endif
  using namespace Util;

  IntVec keep_states(n_states);
  for (int i = 0; i < n_states; ++i) {
    keep_states[i] = 0;
  }
  for (int i = 0; i < n_reduced_states; ++i) {
    if (reduced_to_expanded[i] < 0 || reduced_to_expanded[i] >= n_states) {
    	Util::ThrowError("GetReducedMap : reduced_to_expanded[%d] out "
    			             "of bounds (%d)\n", i, reduced_to_expanded[i]);
    }
    keep_states[reduced_to_expanded[i]] = 1;
  }

  int idx = 0, map_pos = 0, red_len = 0;
  while (map_pos < l2r_map_len) {
    int len = l2r_map[map_pos];

    if (keep_states[idx]) {
      red_len += len + 1;
      if (l2r_map_new) {
        *l2r_map_new = len;
        ++l2r_map_new;
      }
    }

    if(map_pos + len >= l2r_map_len){
    	Util::ThrowError("AdmixBnAbsorbFromLeft : Error in L2R map \n");
    } else {
      if (l2r_map_new && keep_states[idx]) {
        int * mp = l2r_map + (map_pos + 1);
        for(int i = 0; i < len; ++i, ++mp, ++l2r_map_new){
          *l2r_map_new = reduced_map[*mp];
        }
      }
      map_pos += len + 1;
    }
    ++idx;
  }
  if (l2r_map_new_len) *l2r_map_new_len = red_len;

#ifdef CONSOLE_STACK_TRACE
  Util::ConsolePrint("GetReducedMap : Exiting\n");
#endif

}

void FillForwardBackwardMapping(int map_len, int win_sz,
		                            const vector<UnphApproxState>& buf,
		                            bool left_to_right, int* mapping) {
#ifdef EXACT_INF_DEBUG_CHECKS
  Util::LogToFileSprintf("C:/Users/n/Documents/Rpackages/log.txt", 0,
                     "FillForwardBackwardMapping: entering\n");
#endif

  int posn = 0;
  for (int i = 0, sz = buf.size(); i < sz; ++i) {
    int * n_elt = &mapping[posn];
    *n_elt = 0;
    ++posn;
    if (posn >= map_len) {
    	Util::ThrowError("FillForwardBackwardMapping: miscalculated buffer sizes\n");
    }

    for (int j = 0; j < sz; ++j) {
    	int idx1 = left_to_right ? j : i;
    	int idx2 = left_to_right ? i : j;
      if (UnphApproxState::EltsCompat(win_sz, buf[idx1], buf[idx2])) {
        if (posn >= map_len) {
        	Util::ThrowError("FillForwardBackwardMapping: miscalculated buffer sizes\n");
        }
        *n_elt += 1;
        mapping[posn++] = j;
      }
    }
    if (*n_elt == 0) {
      buf[i].ConsolePrint();
    }
  }

#ifdef EXACT_INF_DEBUG_CHECKS
  Util::LogToFileSprintf("C:/Users/n/Documents/Rpackages/log.txt", 0,
                     "FillForwardBackwardMapping: exiting\n");
#endif
}

}  // namespace
}  // namespace

extern "C" {

SEXP UnphApproxStatesSxp_RI(SEXP nGroupsSxp, SEXP winSzSxp, SEXP reducedSxp,
		                        SEXP retSxp) {
	using namespace Util;
	using namespace Aradmx::UnphStatemat;
  int winSz = GetInt(winSzSxp, 0, NULL);
  int nGroups = GetInt(nGroupsSxp, 0, NULL);
  int reduced = GetInt(reducedSxp, 0, NULL);

  if (LENGTH(retSxp) < 3) {
  	Util::ThrowError("UnphApproxStatesSxp_RI: LENGTH(retSxp) < 3\n");
  }

  int n_elt = -1;
  SEXP stateMatSxp = R_NilValue;
  vector<UnphApproxState> buf;
  UnphApproxFillSxp(reduced, nGroups, winSz, &n_elt, &buf, &stateMatSxp);

#ifdef EXACT_INF_DEBUG_CHECKS
  Util::LogToFileSprintf("C:/Users/n/Documents/Rpackages/log.txt", 0,
                     "UnphApproxStatesSxp_RI: check compatibility\n");
#endif

  int n_compat = 0;
  for (int i = 0; i < n_elt; ++i) {
    for (int j = 0; j < n_elt; ++j) {
      n_compat += UnphApproxState::EltsCompat(winSz, buf[i], buf[j]);
    }
  }

#ifdef EXACT_INF_DEBUG_CHECKS
  Util::LogToFileSprintf("C:/Users/n/Documents/Rpackages/log.txt", 0,
                     "UnphApproxStatesSxp_RI: allocating return values\n");
#endif

  int map_len = n_elt + n_compat;
  SEXP L2RMapSxp, R2LMapSxp;
  PROTECT(L2RMapSxp = NEW_INTEGER(map_len));
  PROTECT(R2LMapSxp = NEW_INTEGER(map_len));


#ifdef EXACT_INF_DEBUG_CHECKS
  Util::LogToFileSprintf("C:/Users/n/Documents/Rpackages/log.txt", 0,
                     "UnphApproxStatesSxp_RI: filling mapping\n");
  Util::LogToFileSprintf("C:/Users/n/Documents/Rpackages/log.txt", 0,
                     "map_len = %d, winSz = %d, buf.size() = %d\n",
                     map_len, winSz, (int)buf.size());
#endif

  FillForwardBackwardMapping(map_len, winSz, buf, true, INTEGER(L2RMapSxp));
  FillForwardBackwardMapping(map_len, winSz, buf, false, INTEGER(R2LMapSxp));

  if (LENGTH(retSxp)) {
    SET_VECTOR_ELT(retSxp, 0, stateMatSxp);
  }
  if (LENGTH(retSxp) > 1) {
    SET_VECTOR_ELT(retSxp, 1, L2RMapSxp);
  }
  if (LENGTH(retSxp) > 2) {
    SET_VECTOR_ELT(retSxp, 2, R2LMapSxp);
  }
  UNPROTECT(3);

#ifdef EXACT_INF_DEBUG_CHECKS
  Util::LogToFileSprintf("C:/Users/n/Documents/Rpackages/log.txt", 0,
                     "UnphApproxStatesSxp_RI: exiting\n");
#endif
  return R_NilValue;
}

SEXP ReduceUnphApproxStatesSxp_RI(SEXP stateMat, SEXP l2rMapSxp,
		                                SEXP r2lMapSxp, SEXP retSxp) {
#ifdef CONSOLE_STACK_TRACE
  Util::ConsolePrint("ReduceUnphApproxStatesSxp_RI : Entering\n");
#endif
  using namespace Util;
	using namespace Aradmx::UnphStatemat;

  if (LENGTH(retSxp) < 3) {
  	Util::ThrowError("ReduceUnphApproxStatesSxp_RI: LENGTH(retSxp) < 3\n");
  } else if (!IS_INTEGER(r2lMapSxp) || !IS_INTEGER(l2rMapSxp)) {
  	Util::ThrowError("ReduceUnphApproxStatesSxp_RI: !IS_INTEGER(r2lMapSxp) || !IS_INTEGER(l2rMapSxp)\n");
  } else if (!IS_INTEGER(stateMat)) {
  	Util::ThrowError("ReduceUnphApproxStatesSxp_RI: !IS_INTEGER(stateMat)\n");
  }

  int sttmat_ncol = -1, n_states = -1;
  GetMatrixDim(stateMat, &sttmat_ncol, &n_states);

  if (sttmat_ncol != UnphApproxState::kStateSize) {
  	Util::ThrowError("ReduceUnphApproxStatesSxp_RI: nrow(stateMat) != 5\n");
  }
#ifdef EXACT_INF_DEBUG_CHECKS
  Util::LogToFileSprintf("C:/Users/n/Documents/Rpackages/log.txt", 0,
                     "sttmat_ncol = %d, n_states = %d\n", sttmat_ncol, n_states);
#endif

  int * state_mat_int = INTEGER(stateMat);
  vector<UnphApproxState> state_mat
      = UnphApproxState::StatesFromBuf(state_mat_int, n_states);

  IntVec smap(n_states);
  IntVec reduced_map(n_states);

#ifdef EXACT_INF_DEBUG_CHECKS
  Util::LogToFileSprintf("C:/Users/n/Documents/Rpackages/log.txt", 0,
                     "n_states = %d, state_mat.size() = %d\n",
                     n_states, (int)state_mat.size());
#endif
  UnphApproxState::GetMirrorStates(state_mat, &(smap[0]));
  int n_states_r = NumMirrorStates(n_states, &(smap[0]), &(reduced_map[0]), NULL);

  for (int i = 0, sz = smap.size(); i < sz; ++i) {
  	if (smap[i] < 0) Util::ThrowError("state %d had no mirror", i);
  }

  IntVec reduced_to_expanded(n_states_r);
  NumMirrorStates(n_states, &smap[0], &reduced_map[0], &reduced_to_expanded[0]);

  int * r2l_map = INTEGER(r2lMapSxp);
  int r2l_map_len = LENGTH(r2lMapSxp);
  int * l2r_map = INTEGER(l2rMapSxp);
  int l2r_map_len = LENGTH(l2rMapSxp);


  int new_l2r_len = -1, new_r2l_len = -1;
  GetReducedMap(n_states, l2r_map, l2r_map_len,
                &(reduced_map[0]), n_states_r, &(reduced_to_expanded[0]),
                NULL, &new_l2r_len);
  GetReducedMap(n_states, r2l_map, r2l_map_len,
  		          &(reduced_map[0]), n_states_r, &(reduced_to_expanded[0]),
                NULL, &new_r2l_len);

  SEXP L2RMapSxp, R2LMapSxp;
  PROTECT(L2RMapSxp = NEW_INTEGER(new_l2r_len));
  PROTECT(R2LMapSxp = NEW_INTEGER(new_r2l_len));

  int * new_l2r = INTEGER(L2RMapSxp);
  int * new_r2l = INTEGER(R2LMapSxp);
  GetReducedMap(n_states, l2r_map, l2r_map_len,
  		          &(reduced_map[0]), n_states_r, &(reduced_to_expanded[0]),
                new_l2r, &new_l2r_len);
  GetReducedMap(n_states, r2l_map, r2l_map_len,
  		          &(reduced_map[0]), n_states_r, &(reduced_to_expanded[0]),
                new_r2l, &new_r2l_len);


  vector<UnphApproxState> state_mat_r(n_states_r);
  for (int i = 0; i < n_states; ++i) {
    if (reduced_map[i] < 0 || reduced_map[i] >= n_states_r) {
    	Util::ThrowError("ReduceUnphApproxStatesSxp_RI : reduced_map[%d] = %d",
      		             i, reduced_map[i]);
    }
    state_mat_r[reduced_map[i]] = state_mat[i];
  }

  SEXP stateMatSxp;
  PROTECT(stateMatSxp = NEW_INTEGER(UnphApproxState::kStateSize * n_states_r));
  SetMatrixDim(stateMatSxp, UnphApproxState::kStateSize, n_states_r);
  int * sm = INTEGER(stateMatSxp);

  for (int i = 0; i < n_states_r; ++i) {
  	state_mat_r[i].ToBuf(sm);
    sm += UnphApproxState::kStateSize;
  }

  if (LENGTH(retSxp)) {
    SET_VECTOR_ELT(retSxp, 0, stateMatSxp);
  }
  if (LENGTH(retSxp) > 1) {
    SET_VECTOR_ELT(retSxp, 1, L2RMapSxp);
  }
  if (LENGTH(retSxp) > 2) {
    SET_VECTOR_ELT(retSxp, 2, R2LMapSxp);
  }
  UNPROTECT(3);
#ifdef CONSOLE_STACK_TRACE
  Util::ConsolePrint("ReduceUnphApproxStatesSxp_RI : Exiting\n");
#endif

  return R_NilValue;
}

}
